package de.hillenbrand.datatypes;

import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AgingHashtable<K, V> extends Dictionary<K, V>
   {
   private static final ArrayList<AgingHashtable> instances = new ArrayList<AgingHashtable>();
   private final Hashtable<TimeStampedElement<K>, V> data;
   private final long maxAge;

   static
	{
	final Thread thread = new Thread()
	   {
	   public void run()
		{
		setName("JavaGPE-AgingHashtableElementRemovalThread");
		setPriority(MIN_PRIORITY);

		while (true)
		   {
		   try
			{
			Thread.sleep(1000);
			for (AgingHashtable h : instances)
			   {
			   h.removeAgedElements();
			   yield();
			   }
			}
		   catch (InterruptedException ex)
			{
			Logger.getLogger(AgingHashtable.class.getName()).log(Level.SEVERE, null, ex);
			}
		   }
		}
	   };
	thread.start();
	}

   /**
    * Creates a hashtable where the keys have a limited lifetime.
    * If they are new inserted again within the defined period, they will be removed.
    * @param maxAgeInSeconds the maximum age in seconds
    */
   public AgingHashtable(long maxAgeInSeconds)
	{
	this.data = new Hashtable<TimeStampedElement<K>, V>();
	this.maxAge = Math.max(1, maxAgeInSeconds);
	instances.add(this);
	}

   @Override
   protected void finalize() throws Throwable
	{
	instances.remove(this);
	super.finalize();
	}

   @Override
   public int size()
	{
	synchronized (data)
	   {
	   return data.size();
	   }
	}

   /**
    * Removes all entries at once.
    */
   public void clear()
	{
	synchronized (data)
	   {
	   data.clear();
	   }
	}

   @Override
   public boolean isEmpty()
	{
	synchronized (data)
	   {
	   return data.isEmpty();
	   }
	}

   @Override
   public Enumeration<K> keys()
	{
	synchronized (data)
	   {
	   final Vector<K> result = new Vector<K>();
	   for (TimeStampedElement<K> e : data.keySet())
		{
		result.add(e.getData());
		}
	   return result.elements();
	   }
	}

   @Override
   public Enumeration<V> elements()
	{
	synchronized (data)
	   {
	   return data.elements();
	   }
	}

   @Override
   public V get(Object key)
	{
	synchronized (data)
	   {
	   for (TimeStampedElement<K> e : data.keySet())
		{
		if (e.getData().equals(key))
		   {
		   return data.get(e);
		   }
		}
	   return null;
	   }
	}

   @Override
   public V put(K key, V value)
	{
	synchronized (data)
	   {
	   // update old entry if it exists
	   for (TimeStampedElement<K> e : data.keySet())
		{
		if (e.getData().equals(key))
		   {
		   e.youth();
		   return data.put(e, value);
		   }
		}
	   // otherwise create a new entry
	   final TimeStampedElement<K> tsKey = new TimeStampedElement<K>(key);
	   return data.put(tsKey, value);
	   }
	}

   @Override
   public V remove(Object key)
	{
	synchronized (data)
	   {
	   for (TimeStampedElement<K> e : data.keySet())
		{
		if (e.getData().equals(key))
		   {
		   return data.remove(e);
		   }
		}
	   }
	return null;
	}

   /**
    * Removes all elements from the hashtable that have not been updated in time.
    */
   protected void removeAgedElements()
	{
	synchronized (data)
	   {
	   final long currentTime = System.currentTimeMillis();
	   final Vector<TimeStampedElement<K>> toRemove = new Vector<TimeStampedElement<K>>();
	   for (TimeStampedElement<K> e : data.keySet())
		{
		if (currentTime - e.getTime() > maxAge * 1000)
		   {
		   toRemove.add(e);
		   }
		}

	   if (toRemove.size() > 0)
		{
		for (TimeStampedElement<K> e : toRemove)
		   {
		   data.remove(e);
		   }
		}
	   }
	}

   /**
    * Just a test.
    * @param args
    */
   public static void main(String[] args)
	{
	final Random random = new Random();
	final AgingHashtable<String, String> data = new AgingHashtable<String, String>(5000);
	for (int i = 0; i < 1000; ++i)
	   {
	   data.put("String " + i, "Value " + i);
	   try
		{
		System.out.println(data.size());
		Thread.sleep(random.nextInt(200));
		}
	   catch (InterruptedException ex)
		{
		Logger.getLogger(AgingHashtable.class.getName()).log(Level.SEVERE, null, ex);
		}
	   }
	}
   }
