/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.io;

import java.io.*;
import java.util.concurrent.*;
import de.hillenbrand.datatypes.*;

/**
 * A class that notifies when a file (or a directory) has been changed.
 */
public class FileNotification
   {
   public static int DEFAULT_INTERVAL = 10000; // ms
   // delay queue
   final static private DelayQueue<DelayedQueueEntry> queue = new DelayQueue<DelayedQueueEntry>();
   final static private CheckThread thread = new CheckThread();
   // data
   long lastModified = 0;
   boolean running = true;
   FileChangeListener listener = null;
   // Property: file
   protected File file;
   // Property: interval
   protected long interval = DEFAULT_INTERVAL;

   protected final static class CheckThread extends Thread
	{
	public CheckThread()
	   {
	   setName("JavaGPE-FileNotificationCheck");
	   setPriority(NORM_PRIORITY);
	   }

	public void check(DelayedQueueEntry e)
	   {
	   final FileNotification fileNotification = e.getFileNotification();
	   final File file = fileNotification.getFile();

	   // check
	   if (file.lastModified() > fileNotification.lastModified)
		{
		fileNotification.lastModified = file.lastModified();
		fileNotification.listener.fileChanged(file);
		}

	   // and put into the queue again
	   e.enableDelay(e.getFileNotification().getInterval() / 1000L);
	   queue.add(e);
	   }

	public void run()
	   {
	   while (true)
		{
		try
		   {
		   final DelayedQueueEntry e = queue.take();
		   check(e);
		   }
		catch (InterruptedException e)
		   {
		   System.err.println(e);
		   }
		}
	   }
	}

   protected final static class DelayedQueueEntry extends AbstractDelayedQueueEntry
	{
	// Property: fileNotification
	protected FileNotification fileNotification;

	public DelayedQueueEntry(FileNotification fn)
	   {
	   this.fileNotification = fn;
	   }

	/**
	 * Returns the value of the fileNotification property.
	 * @return
	 */
	public FileNotification getFileNotification()
	   {
	   return fileNotification;
	   }
	}

   public FileNotification(File f, FileChangeListener l)
	{
	this.setFile(f);
	this.listener = l;

	// add to queue
	final DelayedQueueEntry e = new DelayedQueueEntry(this);
	e.enableDelay(interval / 1000L);
	queue.add(e);
	}

   public FileNotification(String filename, FileChangeListener l)
	{
	this(new File(filename), l);
	}

   static
	{
	thread.start();
	}

   /**
    * Returns the value of the file property.
    */
   public File getFile()
	{
	return file;
	}

   /**
    * Returns the interval (in ms) between the file checks.
    */
   public long getInterval()
	{
	return interval;
	}

   /**
    * Assigns the value of the file property.
    */
   public void setFile(File f)
	{
	file = f;
	lastModified = file.lastModified();
	}

   /**
    * Sets the interval (in ms) between the file checks.
    */
   public void setInterval(long ms)
	{
	interval = ms;
	}
   }
