/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.help.*;
import javax.swing.*;
import javax.swing.border.Border;

/**
 * This is the abstract base class for all requesters.
 *
 * @author	Markus Hillenbrand
 */
public abstract class AbstractRequester
   {
   public static final int CANCEL = -1;
   public static final int OK = 0;
   // private things
   protected ActionListener listener = null;
   protected HDialog dialog = null;
   private Dimension size = null;
   private Point location = null;
   // for JavaHelp
   private HelpBroker helpBroker = null;
   private String helpID = null;
   private HelpSet helpSet = null;
   // Property: title
   protected String title = "JReqTools: AbstractRequester";
   // Property: message
   protected String message = "n/a";
   private Border border = null;

   public AbstractRequester(String title, String message)
	{
	this.title = title;
	this.message = message;
	}

   protected static boolean checkForClass(String cname)
	{
	try
	   {
	   Class.forName(cname);
	   return true;
	   }
	catch (Exception x)
	   {
	   return false;
	   }
	}

   protected abstract JPanel createContentPane();

   /**
    * Enables the help key for Java help on the root panel of the dialog.
    * This makes a help set pop up when the user presses F1 (or similar) to get help.
    * Please refer to the JavaHelp documentation for further information.
    *
    * @param	br	the help broker to use
    * @param	id	the id to use when opening the help frame
    * @param	set	the help set to use (null is the default help set)
    */
   public void enableHelpKey4JavaHelp(HelpBroker br, String id, HelpSet set)
	{
	this.helpBroker = br;
	this.helpID = id;
	this.helpSet = set;
	}

   public Rectangle getBounds()
	{
	return location == null || size == null
		  ? new Rectangle(-1, -1, -1, -1)
		  : new Rectangle(location.x, location.y, size.width, size.height);
	}

   protected JButton getDefaultButton()
	{
	return null;
	}

   public Point getLocation()
	{
	return location == null ? new Point(-1, -1) : location;
	}

   /**
    * Returns the value of the message property.
    * @return
    */
   public String getMessage()
	{
	return message;
	}

   protected abstract int getReturnValue();

   public Dimension getSize()
	{
	return size == null ? new Dimension(0, 0) : size;
	}

   /**
    * Returns the value of the title property.
    * @return
    */
   public String getTitle()
	{
	return title;
	}

   /**
    * This method is being called directly before the dialog will be shown.
    */
   protected abstract void prepareForRequest();

   /**
    * Show the requester.
    * @param parent
    * @return
    */
   public int request(JComponent parent)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   return request_(iframe);
	   }
	else
	   {
	   return request_(SwingUtils.getJFrame(parent));
	   }
	}

   public int request(JDialog parent)
	{
	return request_((JFrame) null);
	}

   public int request(JFrame parent)
	{
	return request_(parent);
	}

   private int request_(JFrame parent)
	{
	dialog = new HDialog(parent, getTitle());

	final JPanel content = createContentPane();
	if (border != null)
	   {
	   content.setBorder(border);
	   }

	dialog.setContentPane(content);
	dialog.setDefaultButton(getDefaultButton());

	prepareForRequest();

	if (size != null)
	   {
	   _checkSize(parent, size);
	   dialog.setSize(size);
	   }
	if (location != null)
	   {
	   _checkLocation(parent, location, size);
	   dialog.setLocation(location);
	   }

	dialog.setVisible(true);

	location = dialog.getLocation();
	size = dialog.getSize();

	dialog.dispose();
	dialog = null;

	return getReturnValue();
	}

   private int request_(JInternalFrame parent)
	{
	dialog = new HDialog(parent, getTitle());

	final JPanel content = createContentPane();
	if (border != null)
	   {
	   content.setBorder(border);
	   }

	dialog.setContentPane(content);
	dialog.setDefaultButton(getDefaultButton());

	prepareForRequest();

	if (size != null)
	   {
	   _checkSize(parent, size);
	   dialog.setSize(size);
	   }
	if (location != null)
	   {
	   _checkLocation(parent, location, size);
	   dialog.setLocation(location);
	   }

	dialog.setVisible(true);

	size = dialog.getSize();
	location = dialog.getLocation();

	dialog.dispose();
	dialog = null;

	return getReturnValue();
	}

   public void setBorder(Border border)
	{
	this.border = border;
	}

   public void setBounds(Rectangle newBounds)
	{
	setLocation(newBounds.x, newBounds.y);
	setSize(newBounds.width, newBounds.height);
	}

   public void setBounds(int x, int y, int width, int height)
	{
	setLocation(x, y);
	setSize(width, height);
	}

   public void setLocation(Point newlocation)
	{
	location = (newlocation == null || newlocation.x == -1) ? null : newlocation;
	}

   public void setLocation(int x, int y)
	{
	location = x == -1 ? null : new Point(x, y);
	}

   /**
    * Assigns the value of the message property.
    * @param arg
    */
   public void setMessage(String arg)
	{
	message = arg;
	}

   public void setSize(Dimension newsize)
	{
	size = (newsize == null || newsize.width == 0) ? null : newsize;
	}

   public void setSize(int width, int height)
	{
	size = width == 0 ? null : new Dimension(width, height);
	}

   /**
    * Assigns the value of the title property.
    * @param arg 
    */
   public void setTitle(String arg)
	{
	title = arg;
	}

   public void setVisible(boolean b)
	{
	dialog.setVisible(b);
	}

   public void showError(String text)
	{
	dialog.showError(text);
	}

   public void showException(Throwable t)
	{
	dialog.showException(t);
	}

   public void showMessage(String text)
	{
	dialog.showMessage(text);
	}

   protected void _checkSize(JFrame parent, Dimension size)
	{
	size.width = Math.min(Math.max(size.width, 400), Toolkit.getDefaultToolkit().getScreenSize().width);
	size.height = Math.min(Math.max(size.height, 300), Toolkit.getDefaultToolkit().getScreenSize().height);
	}

   protected void _checkLocation(JFrame parent, Point location, Dimension size)
	{
	if (size == null)
	   {
	   size = new Dimension(400, 300);
	   }

	location.x = Math.min(Math.max(location.x, 0), Toolkit.getDefaultToolkit().getScreenSize().width - size.width);
	location.y = Math.min(Math.max(location.y, 0), Toolkit.getDefaultToolkit().getScreenSize().height - size.height);
	}

   protected void _checkSize(JInternalFrame parent, Dimension size)
	{
	final JDesktopPane desktop = SwingUtils.getJDesktopPane(parent);
	if (desktop == null)
	   {
	   size.width = Math.min(Math.max(size.width, 400), Toolkit.getDefaultToolkit().getScreenSize().width);
	   size.height = Math.min(Math.max(size.height, 300), Toolkit.getDefaultToolkit().getScreenSize().height);
	   }
	else
	   {
	   size.width = Math.min(Math.max(size.width, 400), desktop.getSize().width);
	   size.height = Math.min(Math.max(size.height, 300), desktop.getSize().height);
	   }
	}

   protected void _checkLocation(JInternalFrame parent, Point location, Dimension size)
	{
	if (size == null)
	   {
	   size = new Dimension(400, 300);
	   }

	final JDesktopPane desktop = SwingUtils.getJDesktopPane(parent);
	if (desktop == null)
	   {
	   location.x = 10;
	   location.y = 10;
	   }
	else
	   {
	   location.x = Math.min(Math.max(location.x, 0), desktop.getSize().width - size.width);
	   location.y = Math.min(Math.max(location.y, 0), desktop.getSize().height - size.height);
	   }
	}
   }

