/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

/**
 * A requester for simple strings.
 *
 * @author	Markus Hillenbrand
 */
public class FileRequester extends OkCancelRequester
   {
   private static final boolean DEBUG = false;
   // Variables
   private static String defaultMessage1 = JReqTools.getResourceString("FileRequester.DefaultMessage1");
   private static String defaultMessage2 = JReqTools.getResourceString("FileRequester.DefaultMessage2");
   // locations
   private static Vector<Location> locations = new Vector<Location>();
   // components
   private final HFileChooser chooser = new HFileChooser();
   private final JCheckBox hidden = new JCheckBox(JReqTools.getResourceString("FileRequester.ShowHiddenFiles"));
   // Property: file
   protected File file = null;
   // Property: multiSelection
   protected boolean multiSelection = false;
   // Property: directoriesOnly
   protected boolean directoriesOnly = false;
   // Property: loadMode
   protected boolean loadMode = true;
   // Property: fileHidingEnabled
   protected boolean fileHidingEnabled = true;
   // Property: ensureSuffix
   protected boolean ensureSuffix = false;

   public static class Location implements Serializable
	{
	// properties
	private File directory = null;
	private Icon icon = null;
	private String iconName = null;
	private String description = null;
	// components
	private JButton button = null;
	private JLabel label = null;
	// borders
	private static final Border border1 = new Border()
	   {
	   public Insets getBorderInsets(Component c)
		{
		return new Insets(4, 4, 4, 4);
		}

	   public boolean isBorderOpaque()
		{
		return true;
		}

	   public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
		{
		--w;
		--h;
		g.setColor(Color.black);
		g.drawLine(x, y, x, y + h);
		g.drawLine(x, y, x + w, y);
		g.setColor(Color.white);
		g.drawLine(x + w, y, x + w, y + h);
		g.drawLine(x, y + h, x + w, y + h);
		}
	   };
	private static final Border border2 = BorderFactory.createEmptyBorder(4, 4, 4, 4);

	public Location(String directory, String icon, String description)
	   {
	   setDirectory(directory);
	   setIconName(icon);
	   setDescription(description);
	   }

	public void addActionListener(ActionListener al)
	   {
	   if (button != null)
		{
		button.addActionListener(al);
		}
	   }

	/**
	 * Returns the value of the description property.
	 *
	 * @return
	 */
	public String getDescription()
	   {
	   return description;
	   }

	public File getDirectory()
	   {
	   return directory;
	   }

	public Icon getIcon()
	   {
	   return icon;
	   }

	/**
	 * Returns the value of the iconName property.
	 *
	 * @return
	 */
	public String getIconName()
	   {
	   return iconName;
	   }

	public JComponent getJComponent()
	   {
	   // init button
	   button = new JButton();
	   button.setIcon(icon);
	   button.setMargin(new Insets(1, 1, 1, 1));
	   button.setFocusPainted(false);
	   button.setBorderPainted(false);
	   button.setToolTipText(JReqTools.getResourceString("FileRequester.Location.JumpTo", directory.getPath()));
	   button.setContentAreaFilled(false);

	   // init label
	   label = new JLabel(description, JLabel.CENTER);
	   label.setFont(FontUtils.resize(UIPreferences.getFont(), -2));

	   // create panel
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();
	   c.insets = new Insets(0, 0, 0, 0);

	   final JPanel result = new JPanel(new GridBagLayout());
	   result.setBorder(border2);

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   result.add(button, c);
	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   result.add(label, c);

	   // focus listener
	   button.addFocusListener(new FocusAdapter()
		{
		public void focusGained(FocusEvent e)
		   {
		   result.setBorder(border1);
		   }

		public void focusLost(FocusEvent e)
		   {
		   result.setBorder(border2);
		   }
		});

	   return result;
	   }

	private void readObject(ObjectInputStream is) throws java.io.IOException, java.lang.ClassNotFoundException
	   {
	   setDirectory((String) is.readObject());
	   setIconName((String) is.readObject());
	   setDescription((String) is.readObject());
	   }

	/**
	 * Assigns the value of the description property.
	 *
	 * @param arg
	 */
	public void setDescription(String arg)
	   {
	   description = arg;
	   }

	public void setDirectory(File arg)
	   {
	   this.directory = arg;
	   }

	public void setDirectory(String arg)
	   {
	   this.directory = new File(arg);
	   }

	/**
	 * Assigns the value of the iconName property.
	 *
	 * @param arg
	 */
	public void setIconName(String arg)
	   {
	   this.iconName = arg;
	   this.icon = HIconFactory.getFolderIcon(iconName);
	   }

	public String toString()
	   {
	   return "FileRequester.Location[directory=" + directory + ",icon=" + iconName + ",description=" + description + "]";

	   }

	private void writeObject(ObjectOutputStream os) throws java.io.IOException
	   {
	   os.writeObject(directory.getPath());
	   os.writeObject(iconName);
	   os.writeObject(description);
	   }
	}

   /**
    * Constructs a new FileRequester.
    */
   public FileRequester()
	{
	this("JReqTools: FileRequester", defaultMessage1, new File(System.getProperty("user.home")));
	}

   /**
    * Constructs a new StringRequester.
    *
    * @param	message - The message to be shown in the requester.
    */
   public FileRequester(String message)
	{
	this("JReqTools: FileRequester", message, new File(System.getProperty("user.home")));
	}

   /**
    * Constructs a new StringRequester.
    *
    * @param	message - The message to be shown in the requester.
    * @param	defaultFile - The initially selected file.
    */
   public FileRequester(String message, File defaultFile)
	{
	this("JReqTools: FileRequester", message, defaultFile);
	}

   /**
    * Constructs a new StringRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	defaultFile - The initially selected file.
    */
   public FileRequester(String title, String message, File defaultFile)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_REQUEST_FILE));

	// init chooser
	chooser.setControlButtonsAreShown(false);
	chooser.setBorder(BorderFactory.createEmptyBorder());
	chooser.setAcceptAllFileFilterUsed(false);
	chooser.addChoosableFileFilter(new FileFilter.AllFilesFileFilter());

	// init this
	setFile(defaultFile);

	// listeners
	chooser.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		if (e.getActionCommand().equals("ApproveSelection"))
		   {
		   simulateOkPressed();
		   }
		}
	   });
	hidden.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		fileHidingEnabled = !hidden.isSelected();
		chooser.setFileHidingEnabled(fileHidingEnabled);
		}
	   });
	}

   static
	{
	loadLocations();

	UIManager.put("FileChooser.saveButtonText", JReqTools.getResourceString("FileChooser.Save"));
	UIManager.put("FileChooser.openButtonText", JReqTools.getResourceString("FileChooser.Open"));
	UIManager.put("FileChooser.cancelButtonText", JReqTools.getResourceString("FileChooser.Cancel"));
	UIManager.put("FileChooser.updateButtonText", JReqTools.getResourceString("FileChooser.Update"));
	UIManager.put("FileChooser.helpButtonText", JReqTools.getResourceString("FileChooser.Help"));

	UIManager.put("FileChooser.saveButtonToolTipText", JReqTools.getResourceString("FileChooser.SaveToolTip"));
	UIManager.put("FileChooser.openButtonToolTipText", JReqTools.getResourceString("FileChooser.OpenToolTip"));
	UIManager.put("FileChooser.cancelButtonToolTipText", JReqTools.getResourceString("FileChooser.CancelToolTip"));
	UIManager.put("FileChooser.updateButtonToolTipText", JReqTools.getResourceString("FileChooser.UpdateToolTip"));
	UIManager.put("FileChooser.helpButtonToolTipText", JReqTools.getResourceString("FileChooser.HelpToolTip"));

	UIManager.put("FileChooser.lookInLabelText", JReqTools.getResourceString("FileChooser.LookIn"));
	UIManager.put("FileChooser.fileNameLabelText", JReqTools.getResourceString("FileChooser.Filename"));
	UIManager.put("FileChooser.filesOfTypeLabelText", JReqTools.getResourceString("FileChooser.FilesOfType"));
	UIManager.put("FileChooser.filesLabelText", JReqTools.getResourceString("FileChooser.Files"));
	UIManager.put("FileChooser.foldersLabelText", JReqTools.getResourceString("FileChooser.Folders"));
	UIManager.put("FileChooser.pathLabelText", JReqTools.getResourceString("FileChooser.Path"));
	UIManager.put("FileChooser.filterLabelText", JReqTools.getResourceString("FileChooser.Filter"));
	UIManager.put("FileChooser.enterFileNameLabelText", JReqTools.getResourceString("FileChooser.EnterFileName"));

	UIManager.put("FileChooser.upFolderToolTipText", JReqTools.getResourceString("FileChooser.UpToolTip"));
	UIManager.put("FileChooser.homeFolderToolTipText", JReqTools.getResourceString("FileChooser.HomeToolTip"));
	UIManager.put("FileChooser.newFolderToolTipText", JReqTools.getResourceString("FileChooser.NewToolTip"));
	UIManager.put("FileChooser.listViewButtonToolTipText", JReqTools.getResourceString("FileChooser.ListViewToolTip"));
	UIManager.put("FileChooser.detailsViewButtonToolTipText", JReqTools.getResourceString("FileChooser.DetailsViewToolTip"));
	}

   public void addChoosableFileFilter(javax.swing.filechooser.FileFilter f)
	{
	chooser.addChoosableFileFilter(f);
	}

   /**
    * @param f 
    * @param makeDefault
    * @deprecated
    */
   public void addChoosableFileFilter(javax.swing.filechooser.FileFilter f, boolean makeDefault)
	{
	addChoosableFileFilter(f);
	}

   /**
    * Makes sure that the suffix will be appended.
    * @param b
    */
   public void ensureSuffix(boolean b)
	{
	ensureSuffix = b;
	}

   private String ensureSuffix(String filename)
	{
	javax.swing.filechooser.FileFilter ff = getSelectedFileFilter();
	if (ff instanceof de.hillenbrand.jreqtools.FileFilter.SuffixFileFilter)
	   {
	   de.hillenbrand.jreqtools.FileFilter.SuffixFileFilter sff = (de.hillenbrand.jreqtools.FileFilter.SuffixFileFilter) ff;
	   filename = FileUtils.ensureSuffix(filename, sff.getSuffix());
	   }
	return filename;
	}

   /**
    * Returns the value of the file property.
    * @return
    */
   public File getFile()
	{
	return file;
	}

   public File[] getFiles()
	{
	return chooser.getSelectedFiles();
	}

   protected JPanel getLocationPanel()
	{
	// get default constraints
	final GridBagConstraints c = UIPreferences.getGridBagConstraints();
	c.insets = new Insets(1, 1, 1, 1);

	// create panel
	final JPanel panel = new JPanel(new GridBagLayout());

	// get new color
	final Color bgcolor = ColorUtils.makeDarker(panel.getBackground(), 8);
	panel.setBackground(bgcolor);

	// add components
	for (int i = 0; i < locations.size(); ++i)
	   {
	   final Location loc = locations.elementAt(i);
	   final JComponent comp = loc.getJComponent();
	   comp.setBackground(bgcolor);
	   // add to panel
	   c.gridx = 0;
	   c.gridy = i;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(comp, c);
	   // add listeners
	   loc.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   final File directory = loc.getDirectory();
		   if (directory.exists() && directory.isDirectory())
			{
			FileRequester.this.setCurrentDirectory(directory);
			}
		   else
			{
			showError("'" + directory + "'\n" + JReqTools.getResourceString("DoesNotExist2"));
			}
		   }
		});
	   }

	// add a panel for the rest
	final JPanel jp = new JPanel();
	jp.setBackground(bgcolor);
	c.gridx = 0;
	c.gridy = locations.size();
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 1;
	c.fill = GridBagConstraints.BOTH;
	panel.add(jp, c);

	return panel;
	}

   public static Location[] getLocations()
	{
	return locations.toArray(new Location[0]);
	}

   /**
    * Returns the currently selected file filter.
    * @return
    */
   public javax.swing.filechooser.FileFilter getSelectedFileFilter()
	{
	return chooser.getFileFilter();
	}

   protected JPanel getUserPanel()
	{
	// get default constraints
	final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	// create panel
	final JPanel panel = new JPanel(new GridBagLayout());

	if (UIManager.getLookAndFeel() instanceof com.sun.java.swing.plaf.windows.WindowsLookAndFeel)
	   {
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(chooser, c);
	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new JPanel(), c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(hidden, c);
	   }
	else
	   {
	   // create scroll pane
	   final JPanel locPanel = getLocationPanel();
	   final JScrollPane sp = new JScrollPane(locPanel);
	   sp.setMinimumSize(new Dimension(locPanel.getPreferredSize().width + 10, Short.MAX_VALUE));
	   // add components
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 2;
	   c.weightx = 0;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.VERTICAL;
	   panel.add(sp, c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(chooser, c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new JPanel(), c);
	   c.gridx = 2;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(hidden, c);
	   }

	return panel;
	}

   /**
    * Returns the value of the directoriesOnly property.
    * @return
    */
   public boolean isDirectoriesOnly()
	{
	return directoriesOnly;
	}

   /**
    * Returns the value of the fileHidingEnabled property.
    * @return
    */
   public boolean isFileHidingEnabled()
	{
	return fileHidingEnabled;
	}

   /**
    * Returns the value of the multiSelection property.
    * @return
    */
   public boolean isMultiSelection()
	{
	return multiSelection;
	}

   private static synchronized void loadLocations()
	{
	if (DEBUG)
	   {
	   Logger.log("FileRequester.loadLocations()");
	   }

	// load prefs
	JReqTools.prefs.add("FileRequester.Locations", locations);

	// convert to array or return default values
	if (locations.isEmpty())
	   {
	   locations.add(new Location(System.getProperty("user.home") + File.separator + "Desktop", "desktop", JReqTools.getResourceString("FileRequester.Location.Desktop")));
	   locations.add(new Location(System.getProperty("user.home"), "home", JReqTools.getResourceString("FileRequester.Location.Home")));
	   locations.add(new Location(SwingUtils.isWindowsSystem() ? "C:\\" : "/", "locked", JReqTools.getResourceString("FileRequester.Location.Root")));
	   locations.add(new Location(SwingUtils.isWindowsSystem() ? "C:\\TEMP" : "/tmp", "temporary", JReqTools.getResourceString("FileRequester.Location.Temp")));
	   }
	}

   protected void ok()
	{
	if (loadMode || directoriesOnly)
	   {
	   file = chooser.getSelectedFile();
	   }
	else
	   {
	   final FileChooserUI ui = chooser.getUI();
	   if (ui instanceof BasicFileChooserUI) // this is granted, see validateData();
		{
		final String filename = ((BasicFileChooserUI) ui).getFileName();
		file = new File(chooser.getCurrentDirectory(), ensureSuffix ? ensureSuffix(filename) : filename);
		}
	   else
		{
		file = null;
		}
	   }
	}

   protected void prepareForRequest()
	{
	// init chooser
	chooser.setDialogType(loadMode ? JFileChooser.OPEN_DIALOG : JFileChooser.SAVE_DIALOG);
	chooser.setMultiSelectionEnabled(multiSelection);
	chooser.setFileHidingEnabled(fileHidingEnabled);
	
	if (directoriesOnly)
	   {
	   chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	   chooser.setSelectedFile(file);
	   }
	else
	   {
	   chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	   if (file.isDirectory())
		{
		chooser.setCurrentDirectory(file);
		}
	   else
		{
		chooser.setSelectedFile(file);
		}
	   }

	// init checkbox
	hidden.setSelected(!fileHidingEnabled);
	}

   private static synchronized void saveLocations()
	{
	if (DEBUG)
	   {
	   Logger.log("FileRequester.saveLocations()");
	   }
	JReqTools.prefs.save();
	}

   public void setCurrentDirectory(File directory)
	{
	chooser.setCurrentDirectory(directory);
	}

   public void setDirectoriesOnly(boolean bool)
	{
	if (bool && multiSelection)
	   {
	   System.err.println(getClass().getName() + ".setDirectoriesOnly(): multi selection and directory selection are not allowed together!");
	   return;
	   }

	// store
	directoriesOnly = bool;

	// check messages
	final String msg = getMessage();
	if (msg != null && (msg.equals(defaultMessage1) || msg.equals(defaultMessage2)))
	   {
	   setMessage(directoriesOnly ? defaultMessage2 : defaultMessage1);
	   }
	}

   /**
    * Assigns the value of the file property.
    * @param arg
    */
   public void setFile(File arg)
	{
	file = arg;
	}

   public void setFileName(String name)
	{
	if (file != null)
	   {
	   file = new File(file.getParent(), name);
	   }
	else
	   {
	   file = new File(System.getProperty("user.home"), name);
	   }
	}

   public void setFile(File directory, String filename)
	{
	file = new File(directory, filename);
	}

   public void setFiles(File[] f)
	{
	chooser.setSelectedFiles(f);
	}

   /**
    * Assigns the value of the fileHidingEnabled property.
    * @param arg
    */
   public void setFileHidingEnabled(boolean arg)
	{
	fileHidingEnabled = arg;
	}

   public void setLoadMode()
	{
	loadMode = true;
	setOkButtonText(JReqTools.getResourceString("Load"));
	}

   public static void setLocations(Location[] newLocations)
	{
	if (DEBUG)
	   {
	   Logger.log("FileRequester.setLocations(): " + newLocations.length + " locations");
	   }

	locations.clear();
	for (int i = 0; i < newLocations.length; ++i)
	   {
	   locations.addElement(newLocations[i]);
	   }

	saveLocations();
	}

   /**
    * Assigns the value of the multiSelection property.
    * @param arg
    */
   public void setMultiSelection(boolean arg)
	{
	multiSelection = arg;
	}

   public void setSaveMode()
	{
	loadMode = false;
	setOkButtonText(JReqTools.getResourceString("Save"));
	}

   public void setTitle(String title)
	{
	chooser.setDialogTitle(title);
	}

   public String validateData()
	{
	if (loadMode)
	   {
	   final File selectedFile = chooser.getSelectedFile();
	   if (directoriesOnly)
		{
		if (selectedFile == null)
		   {
		   return JReqTools.getResourceString("FileRequester.Error.NothingSelected.Directory");
		   }
		else if (selectedFile.isFile())
		   {
		   return JReqTools.getResourceString("FileRequester.Error.MustSelectADirectory");
		   }
		}
	   else
		{
		if (selectedFile == null)
		   {
		   return JReqTools.getResourceString("FileRequester.Error.NothingSelected.File");
		   }
		}
	   return null;
	   }
	else // save mode is not supported when JFileChooser is inside a panel, so this is a hack !!!
	   {
	   final FileChooserUI ui = chooser.getUI();
	   if (ui instanceof BasicFileChooserUI)
		{
		final String filename = ((BasicFileChooserUI) ui).getFileName();
		return null;
		}
	   else
		{
		return "The look and feel you are using is not fully supported.\nThus, I cannot get a file name.\nPlease report to javagpe@markus-hillenbrand.de";
		}
	   }
	}
   }
