/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

/**
* This is a requester for fonts.
* 
* @author	Markus Hillenbrand
*/

public class FontRequester extends OkCancelRequester {
	private static final boolean DEBUG = false;

	private Font 					font 			= new Font("Serif", Font.PLAIN, 12);
	private boolean				fixed			= false;
	private boolean				processEvents	= false;
	
	private static final String []	fontList			= makeFontList();
	private static String []			fixedFontList	= null;
	
	// components
	private JList names 				= new JList(fontList);
	private JList size 					= new JList(new String [] { "6", "8", "10", "12", "14", "16", "18", "20", "24", "36", "48", "72" });
	private JCheckBox italic			= new JCheckBox(JReqTools.getResourceString("FontRequester.Italic"), false);
	private JCheckBox bold			= new JCheckBox(JReqTools.getResourceString("FontRequester.Bold"), false);
	private HTextField entersize 		= new HTextField();
	private HFontDisplay fontshower 	= new HFontDisplay();
	
	/**
	* Constructs a new FontRequester.
	* 
	* @param	message - The message to be shown in the requester.
	* @param	defaultFont - The initial font.
	*/

	public FontRequester(String message, Font defaultFont)  {
		this("JReqTools: FontRequester", message, defaultFont);
	}

	/**
	* Constructs a new FontRequester.
	* 
	* @param	title - Title of the requester.
	* @param	message - The message to be shown in the requester.
	* @param	defaultfont - The initial font.
	*/

	public FontRequester(String title, String message, Font defaultfont)  {
		super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_REQUEST_FONT));
		
		// init variables
		if (defaultfont != null) font = defaultfont;
		
		// init components
		names.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		size.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		entersize.setColumns(3);
		
		// listeners
		size.addListSelectionListener(new ListSelectionListener() { public void valueChanged(ListSelectionEvent e) {
			if (! processEvents) return;
			if (e.getValueIsAdjusting()) return;
			entersize.setText((String)size.getSelectedValue());
			fontshower.setFont(createFont());
		}});
		names.addListSelectionListener(new ListSelectionListener() { public void valueChanged(ListSelectionEvent e) {
			if (! processEvents) return;
			if (e.getValueIsAdjusting()) return;
			fontshower.setFont(createFont());
		}});
		entersize.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) {
			if (! processEvents) return;
			fontshower.setFont(createFont()); 
			size.setSelectedValue(entersize.getText(), true);
		}});
		bold.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) {
			if (! processEvents) return;
			fontshower.setFont(createFont()); 
		}});
		italic.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) {
			if (! processEvents) return;
			fontshower.setFont(createFont()); 
		}});
	}

	private void checkSize()  {
		String s = entersize.getText();
		
		StringChecker c = StringCheckers.getIntegerChecker();
		if (! c.accept(entersize.getText())) 
			{ 
			SwingUtils.beep();
			entersize.setText("12"); 
			return; 
			}
		
		try
			{
			int size = Integer.parseInt(s);
			if (size < 6) { SwingUtils.beep(); entersize.setText("6");  return; }
			if (size > 200) { SwingUtils.beep(); entersize.setText("200");  return; }
			}
		catch (Exception x) { SwingUtils.beep(); entersize.setText("12");  return; }
	}

	private Font createFont()  {
		checkSize();
		
		int style = Font.PLAIN;
		if (italic.isSelected()) style += Font.ITALIC;
		if (bold.isSelected()) style += Font.BOLD;
		
		return new Font((String)names.getSelectedValue(), style, Math.abs(Integer.parseInt(entersize.getText())));
	}

	public Font getFont()  {
		return font;
	}

	public String getFontAsString()  {
		String style = "";
		if (font.isBold() && font.isItalic())
			{
			style = "bi";
			}
		else if (font.isBold())
			{
			style = "b";
			}
		else if (font.isItalic())
			{
			style = "i";
			}
				
		return font.getName() + "/" + font.getSize() + style;
	}

	protected JPanel getUserPanel()  {
		// get default constraints
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// create panel
		final JPanel panel = new JPanel(new GridBagLayout());
		c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=4; c.weightx=10; c.weighty=1; c.fill = GridBagConstraints.BOTH; 
		panel.add(new JScrollPane(names), c);
		
		c.gridx = 1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=1; c.fill = GridBagConstraints.BOTH; 
		panel.add(new JScrollPane(size), c);
		c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(entersize, c);
		c.gridx = 1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(italic, c);
		c.gridx = 1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(bold, c);
		
		c.gridx = 0; c.gridy=4; c.gridwidth=2; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(fontshower, c);
		
		return panel;
	}

	protected static String [] makeFixedFontList()  {
		String [] result = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
		
		final JButton dummy = new JButton();
		final Vector<String> fixed = new Vector<String>();
		
		for (int i=0; i<result.length; ++i)
			{
			Font font = new Font(result[i], Font.PLAIN, 12);
			FontMetrics fm = dummy.getFontMetrics(font);
			if (fm.charWidth('w') == fm.charWidth('i')) fixed.addElement(result[i]);
			}
		
		result = new String[fixed.size()];
		fixed.copyInto(result);
		
		return result;
	}

	protected static String [] makeFontList()  {
		return GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
	}

	protected void ok()  {
		font = createFont();
	}

	protected void prepareForRequest()  {
		processEvents = false;
		
		names.setListData(fixed ? fixedFontList : fontList);
		names.setSelectedValue(font.getName(), true);
		size.setSelectedValue(""+font.getSize(), true);
		bold.setSelected(font.isBold());
		italic.setSelected(font.isItalic());
		entersize.setText(""+font.getSize());
		fontshower.setFont(font);
		
		processEvents = true;
	}

	public void setFixedWidthOnly(boolean bool)  {
		if (fixedFontList == null) fixedFontList = makeFixedFontList();
		fixed = bool;
	}

	public void setFont(Font newfont)  {
		font = newfont;
	}

}
