/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import de.hillenbrand.swing.text.*;
import de.hillenbrand.util.ArrayUtils;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URI;
import javax.swing.*;

/**
 * This is a generic and configurable requester.
 *
 * @author	Markus Hillenbrand
 */
public class GenericOkCancelRequester extends OkCancelRequester
   {
   public static final int TYPE_STRING = 0;
   public static final int TYPE_SHORT = 1;
   public static final int TYPE_INTEGER = 2;
   public static final int TYPE_LONG = 3;
   public static final int TYPE_FLOAT = 4;
   public static final int TYPE_DOUBLE = 5;
   public static final int TYPE_PASSWORD = 6;
   public static final int TYPE_CHOICE = 7;
   public static final int TYPE_EDITABLE_CHOICE = 8;
   public static final int TYPE_TEXT = 9;
   public static final int TYPE_SINGLE_SELECTION = 10;
   public static final int TYPE_MULTI_SELECTION = 11;
   public static final int TYPE_EDITABLE_LIST = 12;
   public static final int TYPE_BYTE = 13;
   public static final int TYPE_DATE = 14;
   public static final int TYPE_FILE = 15;
   public static final int TYPE_DIRECTORY = 16;
   public static final int TYPE_TIME_HHMM = 17;
   public static final int TYPE_TIME_HHMMSS = 18;
   public static final int TYPE_TIME_HHMMSSmmm = 19;
   public static final int TYPE_BOOLEAN = 20;
   public static final int TYPE_URI = 21;
   public static final int TYPE_PHONE = 22;
   private static final float[] WEIGHTY =
	{
	0, // string
	0, // short
	0, // integer
	0, // long
	0, // float
	0, // double
	0, // password
	0, // choice
	0, // editable choice
	1, // text
	1, // single selection
	1, // multi selection
	1, // editable list
	0, // byte
	0, // date
	0, // file
	0, // directory
	0, // time
	0, // time
	0, // time
	0, // boolean
	1, // URI
	1, // phone number
	};
   private static final int[] FILL =
	{
	GridBagConstraints.HORIZONTAL, // string
	GridBagConstraints.HORIZONTAL, // short
	GridBagConstraints.NONE, // integer
	GridBagConstraints.HORIZONTAL, // long
	GridBagConstraints.HORIZONTAL, // float
	GridBagConstraints.HORIZONTAL, // double
	GridBagConstraints.HORIZONTAL, // password
	GridBagConstraints.HORIZONTAL, // choice
	GridBagConstraints.HORIZONTAL, // editable Choice
	GridBagConstraints.BOTH, // text
	GridBagConstraints.BOTH, // single selection
	GridBagConstraints.BOTH, // multi selection
	GridBagConstraints.BOTH, // editable list
	GridBagConstraints.HORIZONTAL, // byte
	GridBagConstraints.HORIZONTAL, // date
	GridBagConstraints.HORIZONTAL, // file
	GridBagConstraints.HORIZONTAL, // directory
	GridBagConstraints.HORIZONTAL, // time
	GridBagConstraints.HORIZONTAL, // time
	GridBagConstraints.HORIZONTAL, // time
	GridBagConstraints.NONE, // boolean
	GridBagConstraints.HORIZONTAL, // URI
	GridBagConstraints.HORIZONTAL, // phone number
	};
   // data
   private final TypeConfig[] types;
   private final JComponent[] components;
   private final JTabbedPane tabbedPane = new JTabbedPane();
   // JAR files
   private static final boolean AXIS_TIME_AVAILABLE = checkForClass("org.apache.axis.types.Time");

   public static class Tab extends TypeConfig
	{
	public Tab(String title)
	   {
	   super(title, TYPE_STRING);
	   }
	}

   public static class TypeConfig
	{
	// Property: label
	protected String label = "Label";
	// Property: type
	protected int type = TYPE_STRING;
	// Property: value
	protected Object value = null;
	// Property: options
	protected Object options = null;
	// Property: renderer
	protected Object renderer = null;

	public TypeConfig(String label, int type)
	   {
	   this.label = label;
	   this.type = type;
	   }

	public TypeConfig(String label, int type, Object defaultValue)
	   {
	   this.label = label;
	   this.type = type;
	   this.value = defaultValue;
	   }

	public TypeConfig(String label, int type, Object defaultValue, Object options)
	   {
	   this.label = label;
	   this.type = type;
	   this.value = defaultValue;
	   this.options = options;
	   }

	public TypeConfig(String label, int type, Object defaultValue, Object options, Object renderer)
	   {
	   this.label = label;
	   this.type = type;
	   this.value = defaultValue;
	   this.options = options;
	   this.renderer = renderer;
	   }

	/**
	 * Returns the value of the label property.
	 *
	 * @return
	 */
	public String getLabel()
	   {
	   return label;
	   }

	/**
	 * Returns the value of the options property.
	 *
	 * @return
	 */
	public Object getOptions()
	   {
	   return options;
	   }

	/**
	 * Returns the value of the renderer property.
	 *
	 * @return
	 */
	public Object getRenderer()
	   {
	   return renderer;
	   }

	/**
	 * Returns the value of the type property.
	 *
	 * @return
	 */
	public int getType()
	   {
	   return type;
	   }

	/**
	 * Returns the value of the value property.
	 *
	 * @return
	 */
	public Object getValue()
	   {
	   return value;
	   }

	/**
	 * Assigns the value of the label property.
	 *
	 * @param arg
	 */
	public void setLabel(String arg)
	   {
	   label = arg;
	   }

	/**
	 * Assigns the value of the options property.
	 *
	 * @param arg
	 */
	public void setOptions(Object arg)
	   {
	   options = arg;
	   }

	/**
	 * Assigns the value of the renderer property.
	 *
	 * @param arg
	 */
	public void setRenderer(Object arg)
	   {
	   renderer = arg;
	   }

	/**
	 * Assigns the value of the type property.
	 *
	 * @param arg
	 */
	public void setType(int arg)
	   {
	   type = arg;
	   }

	/**
	 * Assigns the value of the value property.
	 *
	 * @param arg
	 */
	public void setValue(Object arg)
	   {
	   value = arg;
	   }
	}

   public GenericOkCancelRequester(String message, TypeConfig[] types)
	{
	this("JReqTools: Generic Requester", message, types);
	}

   public GenericOkCancelRequester(String title, String message, TypeConfig[] types)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ENTERDATA));

	this.types = types;
	this.components = new JComponent[types.length];

	setSize(640, 480);
	}

   protected String check(String text, StringChecker checker)
	{
	if (checker.accept(text))
	   {
	   return null;
	   }
	return checker.getFailMessage();
	}

   protected static int[] convertToIntArray(Object o)
	{
	final Integer[] data = (Integer[]) o;
	final int[] result = new int[data.length];
	for (int i = 0; i < data.length; ++i)
	   {
	   result[i] = data[i].intValue();
	   }
	return result;
	}

   protected JComponent createJComponent(TypeConfig tc)
	{
	// TODO: Change to JFormatter where possible
	try
	   {
	   final Object value = tc.getValue();
	   final Object options = tc.getOptions();
	   final Object renderer = tc.getRenderer();

	   switch (tc.getType())
		{
		case TYPE_PHONE:
		   {
		   final RegexFormatter formatter = new RegexFormatter("\\+?[0-9]*?[-,/]?\\(?[0]?\\)?[0-9]*[-,/,.]?[0-9]*[-,/,.]?[0-9]*[-,/,.]?[0-9]*$");
		   final HFormattedTextField result = new HFormattedTextField(formatter);
		   result.setText("" + value);
		   return result;
		   }
		case TYPE_URI:
		   {
		   final RegexFormatter formatter = new RegexFormatter("[a-zA-Z]*://.*");
		   final HFormattedTextField result = new HFormattedTextField(formatter);
		   result.setText("" + value);
		   return result;
		   }
		case TYPE_STRING:
		   if (value instanceof String)
			{
			final HTextField result = new HTextField((String) value);
			final String[] stringOptions = toStringArray(options);
			final JPopupMenu menu = result.getJPopupMenu();
			menu.addSeparator();
			SwingUtils.addMenuItemsFromStringArray(stringOptions, "'", "'", menu, new ActionListener()
			   {
			   public void actionPerformed(ActionEvent e)
				{
				result.setText(e.getActionCommand());
				}
			   });
			return result;
			}
		   break;
		case TYPE_SHORT:
		   if (value instanceof Short)
			{
			return new HTextField("" + (Short) value);
			}
		   if (value instanceof String)
			{
			return new HTextField((String) value);
			}
		   break;
		case TYPE_INTEGER:
		   final Integer[] integerOptions = toIntegerArray(options);
		   final int initial = (value instanceof Integer) ? (Integer) value : (value instanceof String) ? Integer.parseInt((String) value) : 0;
		   final int min = (integerOptions.length > 0) ? integerOptions[0] : Integer.MIN_VALUE;
		   final int max = (integerOptions.length > 1) ? integerOptions[1] : Integer.MAX_VALUE;
		   final int step = (integerOptions.length > 2) ? integerOptions[2] : 1;
		   final SpinnerModel integerModel = new SpinnerNumberModel(initial, min, max, step);
		   final JSpinner integerSpinner = new JSpinner(integerModel);
		   integerSpinner.setEditor(new JSpinner.NumberEditor(integerSpinner, "#"));
		   integerSpinner.setToolTipText(JReqTools.getResourceString("GenericRequester.Integer", min, max));
		   return integerSpinner;
		case TYPE_LONG:
		   if (value instanceof Long)
			{
			return new HTextField("" + (Long) value);
			}
		   if (value instanceof String)
			{
			return new HTextField((String) value);
			}
		   break;
		case TYPE_FLOAT:
		   if (value instanceof Float)
			{
			return new HTextField("" + (Float) value);
			}
		   if (value instanceof String)
			{
			return new HTextField((String) value);
			}
		   break;
		case TYPE_DOUBLE:
		   if (value instanceof Double)
			{
			return new HTextField("" + (Double) value);
			}
		   if (value instanceof String)
			{
			return new HTextField((String) value);
			}
		   break;
		case TYPE_PASSWORD:
		   if (value instanceof char[])
			{
			return new HPasswordField(new String((char[]) value));
			}
		   if (value instanceof String)
			{
			return new HPasswordField((String) value);
			}
		   break;
		case TYPE_CHOICE:
		   if (value instanceof Object[])
			{
			final HComboBox<Object> c = new HComboBox<Object>((Object[]) value);
			if (options != null)
			   {
			   c.setSelectedIndex(((Integer) options).intValue());
			   }
			if (renderer != null && renderer instanceof ListCellRenderer)
			   {
			   c.setRenderer((ListCellRenderer) renderer);
			   }
			return c;
			}
		   break;
		case TYPE_BOOLEAN:
		   if (value instanceof Boolean)
			{
			return new JCheckBox("", (Boolean) value);
			}
		   break;
		case TYPE_EDITABLE_CHOICE:
		   if (value instanceof Object[])
			{
			final HComboBox<Object> c = new HComboBox<Object>((Object[]) value);
			if (options != null)
			   {
			   c.setSelectedIndex(((Integer) options).intValue());
			   }
			if (renderer != null && renderer instanceof ListCellRenderer)
			   {
			   c.setRenderer((ListCellRenderer) renderer);
			   }
			c.setEditable(true);
			return c;
			}
		   break;
		case TYPE_TEXT:
		   if (value instanceof String)
			{
			final HTextArea result = new HTextArea((String) value);

			final String[] stringOptions = toStringArray(options);
			final JPopupMenu menu = result.getJPopupMenu();
			menu.addSeparator();
			SwingUtils.addMenuItemsFromStringArray(stringOptions, "'", "'", menu, new ActionListener()
			   {
			   public void actionPerformed(ActionEvent e)
				{
				final String text = result.getText();
				result.append(text.endsWith("\n") || text.length() == 0 ? e.getActionCommand() : "\n" + e.getActionCommand());
				}
			   });

			return new JScrollPane(result);
			}
		   break;
		case TYPE_SINGLE_SELECTION:
		   if (value instanceof Object[])
			{
			final JList c = new JList((Object[]) value);
			c.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
			if (options != null)
			   {
			   c.setSelectedIndex(((Integer) options).intValue());
			   }
			if (renderer != null && renderer instanceof ListCellRenderer)
			   {
			   c.setCellRenderer((ListCellRenderer) renderer);
			   }
			return new JScrollPane(c);
			}
		   break;
		case TYPE_MULTI_SELECTION:
		   if (value instanceof Object[])
			{
			final JList c = new JList((Object[]) value);
			c.setSelectionMode(DefaultListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			if (options != null)
			   {
			   c.setSelectedIndices(convertToIntArray(options));
			   }
			if (renderer != null && renderer instanceof ListCellRenderer)
			   {
			   c.setCellRenderer((ListCellRenderer) renderer);
			   }
			return new JScrollPane(c);
			}
		   break;
		case TYPE_EDITABLE_LIST:
		   if (value instanceof String[])
			{
			final HEditableList edlist = new HEditableList((String[]) value);
			if (options != null && options instanceof String[])
			   {
			   edlist.createPopupMenuFromStringArray((String[]) options, "hotlistadd");
			   }
			return edlist;
			}
		   break;
		case TYPE_BYTE:
		   if (value instanceof Byte)
			{
			return new HTextField("" + (Byte) value);
			}
		   if (value instanceof String)
			{
			return new HTextField((String) value);
			}
		   break;
		case TYPE_DATE:
		   final com.toedter.calendar.JDateChooser comp = new com.toedter.calendar.JDateChooser();
		   if (value instanceof java.util.Date)
			{
			comp.setDate((java.util.Date) value);
			}
		   else if (value instanceof java.util.Calendar)
			{
			comp.setDate(((java.util.Calendar) value).getTime());
			}
		   else
			{
			throw new RuntimeException("Wrong type for type config. Must be Date or Calendar");
			}
		   return comp;
		case TYPE_TIME_HHMM:
		   if (value instanceof String)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMM, (String) value);
			}
		   else if (value instanceof java.sql.Time)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMM, value.toString().substring(0, 5));
			}
		   else if (AXIS_TIME_AVAILABLE && value instanceof org.apache.axis.types.Time)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMM, value.toString().substring(0, 5));
			}
		   break;
		case TYPE_TIME_HHMMSS:
		   if (value instanceof String)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMMSS, (String) value);
			}
		   else if (value instanceof java.sql.Time)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMMSS, value.toString());
			}
		   else if (AXIS_TIME_AVAILABLE && value instanceof org.apache.axis.types.Time)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMMSS, value.toString().substring(0, 8));
			}
		   break;
		case TYPE_TIME_HHMMSSmmm:
		   if (value instanceof String)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMMSSmmm, (String) value);
			}
		   else if (value instanceof java.sql.Time)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMMSSmmm, value.toString() + ":000");
			}
		   else if (AXIS_TIME_AVAILABLE && value instanceof org.apache.axis.types.Time)
			{
			return new HTimeField(HTimeField.TimeFormat.HHMMSSmmm, value.toString().substring(0, 12).replace('.', ':'));
			}
		   break;
		case TYPE_FILE:
		   // create component
		   final HFileString fsfile = value instanceof File
			     ? new HFileString((File) value, HFileString.FILES_AND_DIRECTORIES)
			     : new HFileString((String) value, HFileString.FILES_AND_DIRECTORIES);
		   // and add file filters if necessary
		   if (options instanceof String)
			{
			final FileFilter.SuffixFileFilter ff = new FileFilter.SuffixFileFilter((String) options, (String) options);
			fsfile.addChoosableFileFilter(ff);
			}
		   else if (options instanceof String[])
			{
			final String[] suffixes = (String[]) options;
			for (int i = 0; i < suffixes.length; ++i)
			   {
			   final FileFilter.SuffixFileFilter ff = new FileFilter.SuffixFileFilter(suffixes[i], suffixes[i]);
			   fsfile.addChoosableFileFilter(ff);
			   }
			}
		   // return component
		   return fsfile;
		case TYPE_DIRECTORY:
		   // create and return component
		   if (value instanceof File)
			{
			return new HFileString((File) value, HFileString.DIRECTORIES_ONLY);
			}
		   if (value instanceof String)
			{
			return new HFileString((String) value, HFileString.DIRECTORIES_ONLY);
			}
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }

	return new JLabel("Cannot create component for type " + tc.getType());
	}

   /**
    * Returns the value of TypeConfig at index.
    *
    * @param	index index of the TypeConfig
    * @return
    */
   public Object getResult(int index)
	{
	try
	   {
	   String s = null;
	   File f = null;
	   Object o = null;
	   Component view = null;

	   if (types[index] instanceof Tab)
		{
		return null;
		}

	   switch (types[index].getType())
		{
		case TYPE_PHONE:
		   s = ((HFormattedTextField) components[index]).getText();
		   return s;
		case TYPE_URI:
		   s = ((HFormattedTextField) components[index]).getText();
		   return new URI(s);
		case TYPE_STRING:
		   s = ((HTextField) components[index]).getText();
		   return s;
		case TYPE_SHORT:
		   s = ((HTextField) components[index]).getText();
		   return new Short(s);
		case TYPE_INTEGER:
		   o = ((JSpinner) components[index]).getValue();
		   return (Integer) o;
		case TYPE_LONG:
		   s = ((HTextField) components[index]).getText();
		   return new Long(s);
		case TYPE_FLOAT:
		   s = ((HTextField) components[index]).getText();
		   return new Float(s);
		case TYPE_DOUBLE:
		   s = ((HTextField) components[index]).getText();
		   return new Double(s);
		case TYPE_PASSWORD:
		   return ((HPasswordField) components[index]).getPassword();
		case TYPE_CHOICE:
		case TYPE_EDITABLE_CHOICE:
		   return ((HComboBox) components[index]).getSelectedItem();
		case TYPE_BOOLEAN:
		   return ((JCheckBox) components[index]).isSelected();
		case TYPE_TEXT:
		   view = ((JScrollPane) components[index]).getViewport().getView();
		   return ((JTextArea) view).getText();
		case TYPE_SINGLE_SELECTION:
		   view = ((JScrollPane) components[index]).getViewport().getView();
		   return ((JList) view).getSelectedValue();
		case TYPE_MULTI_SELECTION:
		   view = ((JScrollPane) components[index]).getViewport().getView();
		   return ((JList) view).getSelectedValues();
		case TYPE_EDITABLE_LIST:
		   return ((HEditableList) components[index]).get();
		case TYPE_BYTE:
		   s = ((HTextField) components[index]).getText();
		   return new Byte(s);
		case TYPE_DATE:
		   final java.util.Date time = ((com.toedter.calendar.JDateChooser) components[index]).getDate();
		   if (types[index].value instanceof java.util.Calendar)
			{
			final java.util.Calendar res = java.util.Calendar.getInstance();
			res.setTime(time);
			return res;
			}
		   else if (types[index].value instanceof java.sql.Date)
			{
			final java.sql.Date res = new java.sql.Date(time.getTime());
			return res;
			}
		   else
			{
			return time;
			}
		case TYPE_FILE:
		case TYPE_DIRECTORY:
		   f = ((HFileString) components[index]).getFile();
		   if (types[index].value instanceof String)
			{
			return f.toString();
			}
		   else
			{
			return f;
			}
		case TYPE_TIME_HHMM:
		   s = ((HTimeField) components[index]).getText();
		   if (types[index].value instanceof java.sql.Time)
			{
			return java.sql.Time.valueOf(s + ":00");
			}
		   else if (AXIS_TIME_AVAILABLE && types[index].value instanceof org.apache.axis.types.Time)
			{
			return new org.apache.axis.types.Time(s + ":00");
			}
		   else
			{
			return s;
			}
		case TYPE_TIME_HHMMSS:
		   s = ((HTimeField) components[index]).getText();
		   if (types[index].value instanceof java.sql.Time)
			{
			return java.sql.Time.valueOf(s);
			}
		   else if (AXIS_TIME_AVAILABLE && types[index].value instanceof org.apache.axis.types.Time)
			{
			return new org.apache.axis.types.Time(s);
			}
		   else
			{
			return s;
			}
		case TYPE_TIME_HHMMSSmmm:
		   s = ((HTimeField) components[index]).getText();
		   if (types[index].value instanceof java.sql.Time)
			{
			return java.sql.Time.valueOf(s.substring(0, 8));
			}
		   else if (AXIS_TIME_AVAILABLE && types[index].value instanceof org.apache.axis.types.Time)
			{
			return new org.apache.axis.types.Time(s.substring(0, 8) + "." + s.substring(9, 12));
			}
		   else
			{
			return s;
			}
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }

	throw new RuntimeException("Wrong index or type.");
	}

   protected JPanel getUserPanel()
	{
	boolean useTabbedPane = false;

	// get default constraints
	JPanel panel = new JPanel(new GridBagLayout());
	GridBagConstraints c = UIPreferences.getGridBagConstraints();

	// create panel
	for (int i = 0; i < types.length; ++i)
	   {
	   if (types[i] instanceof GenericOkCancelRequester.Tab)
		{
		useTabbedPane = true;
		panel = new JPanel(new GridBagLayout());
		panel.setBorder(UIPreferences.getFrameBorder());
		tabbedPane.addTab(types[i].getLabel(), panel);
		continue;
		}
	   // create component
	   components[i] = createJComponent(types[i]);
	   // label
	   c.gridx = 0;
	   c.gridy = i;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new JLabel(types[i].getLabel(), JLabel.LEFT), c);
	   // component
	   c.anchor = GridBagConstraints.WEST;
	   c.gridx = 1;
	   c.gridy = i;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = WEIGHTY[types[i].getType()];
	   c.fill = FILL[types[i].getType()];
	   panel.add(components[i], c);
	   }

	if (useTabbedPane)
	   {
	   panel = new JPanel(new GridBagLayout());
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(tabbedPane, c);
	   }

	return panel;
	}

   protected void ok()
	{
	}

   protected void prepareForRequest()
	{
	}

   public void setRenderer(int i, Object renderer)
	{
	types[i].setRenderer(renderer);
	}

   /**
    * Sets the font for the tabbedpane.
    * @param f
    */
   public void setTabFont(Font f)
	{
	tabbedPane.setFont(f);
	}

   /**
    * Sets the tab placement for the tabbedpane.  Tab layout policy may be either: JTabbedPane.WRAP_TAB_LAYOUT or JTabbedPane.SCROLL_TAB_LAYOUT.
    * @param policy
    */
   public void setTabLayoutPolicy(int policy)
	{
	tabbedPane.setTabLayoutPolicy(policy);
	}

   /**
    * Sets the tab placement for the tabbedpane. Tab placement may be either: JTabbedPane.TOP, JTabbedPane.BOTTOM, JTabbedPane.LEFT, or JTabbedPane.RIGHT.
    * @param placement
    */
   public void setTabPlacement(int placement)
	{
	tabbedPane.setTabPlacement(placement);
	}

   /**
    * Sets the value of TypeConfig at index.
    *
    * @param	index index of the TypeConfig
    * @param	value value to assign to that index
    */
   public void setValue(int index, Object value)
	{
	types[index].setValue(value);
	}

   private Integer[] toIntegerArray(Object o)
	{
	if (o == null)
	   {
	   return new Integer[0];
	   }
	if (o instanceof Integer[])
	   {
	   return (Integer[]) o;
	   }
	if (o instanceof String[])
	   {
	   final Integer[] result = new Integer[((String[]) o).length];
	   for (int i = 0; i < ((String[]) o).length; ++i)
		{
		result[i] = Integer.parseInt(((String[]) o)[i]);
		}
	   return result;
	   }

	System.err.println("Wrong integer array type: " + o.getClass().getCanonicalName());
	return new Integer[0];
	}

   private String[] toStringArray(Object o)
	{
	if (o == null)
	   {
	   return new String[0];
	   }
	if (o instanceof String)
	   {
	   return ArrayUtils.toArray((String) o);
	   }
	if (o instanceof String[])
	   {
	   return (String[]) o;
	   }
	if (o instanceof Object[])
	   {
	   final String[] result = new String[((Object[]) o).length];
	   for (int i = 0; i < ((Object[]) o).length; ++i)
		{
		result[i] = "" + ((Object[]) o)[i];
		}
	   return result;
	   }

	System.err.println("Wrong string array type: " + o.getClass().getCanonicalName());
	return new String[0];
	}

   protected String validateData()
	{
	for (int i = 0; i < types.length; ++i)
	   {
	   switch (types[i].getType())
		{
		case TYPE_BYTE:
		   return check(((HTextField) components[i]).getText(), StringCheckers.getByteChecker());
		case TYPE_SHORT:
		   return check(((HTextField) components[i]).getText(), StringCheckers.getShortChecker());
		//case TYPE_INTEGER:	return check(((HTextField)components[i]).getText(), StringCheckers.getIntegerChecker());
		case TYPE_LONG:
		   return check(((HTextField) components[i]).getText(), StringCheckers.getLongChecker());
		case TYPE_DOUBLE:
		   return check(((HTextField) components[i]).getText(), StringCheckers.getDoubleChecker());
		case TYPE_FLOAT:
		   return check(((HTextField) components[i]).getText(), StringCheckers.getFloatChecker());
		}
	   }

	return null;
	}
   }
