/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * This is a requester for look and feels. The chosen look and feel can also be set immediately.
 *
 * @author	Markus Hillenbrand
 */
public class LookAndFeelRequester extends SelectRequester
   {
   private Component set = null;

   public static class LookAndFeelInfo extends UIManager.LookAndFeelInfo implements Comparable<LookAndFeelInfo>
	{
	private LookAndFeel lf = null;
	private Icon icon = null;

	public LookAndFeelInfo(UIManager.LookAndFeelInfo info)
	   {
	   this(info.getName(), info.getClassName());
	   }

	public LookAndFeelInfo(String name, String classname)
	   {
	   super(name, classname);
	   }

	public int compareTo(LookAndFeelInfo i)
	   {
	   int value = getName().compareTo(i.getName());
	   if (value != 0)
		{
		return value;
		}
	   return getClassName().compareTo(i.getClassName());
	   }

	public boolean equals(LookAndFeelInfo i)
	   {
	   return compareTo(i) == 0;
	   }

	public String getAuthor()
	   {
	   if (lf == null)
		{
		initLF();
		}
	   return (lf != null) && (lf instanceof HLookAndFeelInformation) ? ((HLookAndFeelInformation) lf).getAuthor() : "";
	   }

	public String getCopyright()
	   {
	   if (lf == null)
		{
		initLF();
		}
	   return (lf != null) && (lf instanceof HLookAndFeelInformation) ? ((HLookAndFeelInformation) lf).getCopyright() : "";
	   }

	public String getDescription()
	   {
	   if (lf == null)
		{
		initLF();
		}
	   return lf != null ? lf.getDescription() : "";
	   }

	public String getDownloadUrl()
	   {
	   if (lf == null)
		{
		initLF();
		}
	   return (lf != null) && (lf instanceof HLookAndFeelInformation) ? ((HLookAndFeelInformation) lf).getDownloadUrl() : "";
	   }

	public String getID()
	   {
	   if (lf == null)
		{
		initLF();
		}
	   return lf != null ? lf.getID() : "";
	   }

	public Icon getIcon()
	   {
	   if (icon == null)
		{
		return HIconFactory.getIcon("lookandfeel_" + getID());
		}

	   return icon;
	   }

	private void initLF()
	   {
	   try
		{
		final Class c = Class.forName(getClassName());
		lf = (LookAndFeel) c.newInstance();
		}
	   catch (ClassNotFoundException x1)
		{
		}
	   catch (IllegalAccessException x2)
		{
		}
	   catch (InstantiationException x3)
		{
		}
	   }

	public String toString()
	   {
	   return getName();
	   }
	}

   /**
    * Renders an item in a list.
    * <p>
    * <strong><a name="override">Implementation Note:</a></strong>
    * This class overrides
    * <code>validate</code>,
    * <code>revalidate</code>,
    * <code>repaint</code>,
    * and
    * <code>firePropertyChange</code>
    * solely to improve performance.
    * If not overridden, these frequently called methods would execute code paths
    * that are unnecessary for the default list cell renderer.
    * If you write your own renderer,
    * take care to weigh the benefits and
    * drawbacks of overriding these methods.
    *
    * <p>
    *
    * <strong>Warning:</strong>
    * Serialized objects of this class will not be compatible with
    * future Swing releases. The current serialization support is
    * appropriate for short term storage or RMI between applications running
    * the same version of Swing.  As of 1.4, support for long term storage
    * of all JavaBeans<sup><font size="-2">TM</font></sup>
    * has been added to the <code>java.beans</code> package.
    * Please see {@link java.beans.XMLEncoder}.
    *
    * @version 1.21 12/03/01
    * @author Philip Milne
    * @author Hans Muller
    *
    */
   public static class LookAndFeelInfoListCellRenderer extends JPanel implements ListCellRenderer
	{
	private Border noFocusBorder = null;
	private JLabel icon = new JLabel();
	private JLabel name = new JLabel();
	private JLabel description = new JLabel();
	private JLabel classname = new JLabel();
	private JLabel copyright = new JLabel();
	private JLabel author = new JLabel();
	private JLabel downloadurl = new JLabel();
	private JLabel classnameLabel = new JLabel("Class:", JLabel.RIGHT);
	private JLabel copyrightLabel = new JLabel("Copyright:", JLabel.RIGHT);
	private JLabel authorLabel = new JLabel("Author:", JLabel.RIGHT);
	private JLabel downloadurlLabel = new JLabel("Download:", JLabel.RIGHT);

	/**
	 * Constructs a renderer object for a LookAndFeelInfo item in a list.
	 */
	public LookAndFeelInfoListCellRenderer()
	   {
	   super(new GridBagLayout());

	   // create no focus border
	   final Border border = UIManager.getBorder("List.focusCellHighlightBorder");
	   final Insets borderInsets = border.getBorderInsets(this);
	   noFocusBorder = new EmptyBorder(borderInsets.top, borderInsets.right, borderInsets.bottom, borderInsets.left);

	   // init
	   setOpaque(true);
	   setBorder(noFocusBorder);
	   icon.setBorder(BorderFactory.createLineBorder(Color.black));

	   // set title border
	   name.setBorder(new Border()
		{
		public Insets getBorderInsets(Component c)
		   {
		   return new Insets(2, 0, 2, 0);
		   }

		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
		   {
		   --h;
		   --w;
		   g.setColor(Color.black);
		   g.drawLine(x, y, x + w, y);
		   g.drawLine(x, y + h, x + w, y + h);
		   }

		public boolean isBorderOpaque()
		   {
		   return false;
		   }
		});

	   // fonts
	   name.setFont(UIPreferences.getMessageFont());
	   description.setFont(UIPreferences.getFont());
	   copyright.setFont(UIPreferences.getAcceleratorFont());
	   copyrightLabel.setFont(UIPreferences.getAcceleratorFont());
	   classname.setFont(UIPreferences.getFont());
	   classnameLabel.setFont(UIPreferences.getFont());
	   author.setFont(UIPreferences.getAcceleratorFont());
	   authorLabel.setFont(UIPreferences.getAcceleratorFont());
	   downloadurl.setFont(UIPreferences.getAcceleratorFont());
	   downloadurlLabel.setFont(UIPreferences.getAcceleratorFont());

	   // space panels
	   final JPanel space1 = new JPanel();
	   space1.setOpaque(false);
	   final JPanel space2 = new JPanel();
	   space2.setOpaque(false);

	   // create constraints
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();
	   c.insets = new Insets(1, 1, 1, 1);

	   // add components
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 7;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.NONE;
	   this.add(icon, c);

	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(name, c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(description, c);

	   c.gridx = 1;
	   c.gridy = 2;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   this.add(space1, c);

	   c.gridx = 1;
	   c.gridy = 3;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(classnameLabel, c);
	   c.gridx = 2;
	   c.gridy = 3;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(classname, c);
	   c.gridx = 1;
	   c.gridy = 4;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(copyrightLabel, c);
	   c.gridx = 2;
	   c.gridy = 4;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(copyright, c);

	   c.gridx = 1;
	   c.gridy = 5;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(authorLabel, c);
	   c.gridx = 2;
	   c.gridy = 5;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(author, c);

	   c.gridx = 1;
	   c.gridy = 6;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(downloadurlLabel, c);
	   c.gridx = 2;
	   c.gridy = 6;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(downloadurl, c);

	   c.gridx = 1;
	   c.gridy = 7;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   this.add(space2, c);
	   }

	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
	   {
	   setComponentOrientation(list.getComponentOrientation());

	   if (isSelected)
		{
		setBackground(list.getSelectionBackground());
		setForeground(list.getSelectionForeground());
		}
	   else
		{
		setBackground(list.getBackground());
		setForeground(list.getForeground());
		}

	   if (value instanceof LookAndFeelInfo)
		{
		icon.setIcon(((LookAndFeelInfo) value).getIcon());
		name.setText(((LookAndFeelInfo) value).getName());
		classname.setText(((LookAndFeelInfo) value).getClassName());
		description.setText(((LookAndFeelInfo) value).getDescription());
		copyright.setText(((LookAndFeelInfo) value).getCopyright());
		author.setText(((LookAndFeelInfo) value).getAuthor());
		downloadurl.setText(((LookAndFeelInfo) value).getDownloadUrl());
		}
	   else
		{
		name.setText((value == null) ? "n/a" : value.toString());
		description.setText((value == null) ? "n/a" : value.getClass().getName());
		}

	   setEnabled(list.isEnabled());
	   setFont(list.getFont());
	   setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);

	   return this;
	   }
	}

   public LookAndFeelRequester(String message)
	{
	this("JReqTools: LookAndFeelRequester", message);
	}

   public LookAndFeelRequester(String title, String message)
	{
	super(title, message, getInstalledLookAndFeels());

	// set default value
	final LookAndFeel lf = UIManager.getLookAndFeel();
	final LookAndFeelInfo lfi = new LookAndFeelInfo(lf.getName(), lf.getClass().getName());
	setSelectedValue(lfi);

	// set cell renderer
	setListCellRenderer(new LookAndFeelInfoListCellRenderer());

	// set size of frame
	setSize(800, 500);
	}

   static
	{
	UIPreferences.init();
	}

   /**
    * Returns the names of all installed Look & Feels in a sorted array.
    * @return
    */
   public static String[] getInstalledLookAndFeelNames()
	{
	// get all installed l&f and sort them
	final UIManager.LookAndFeelInfo[] info = getInstalledLookAndFeels();

	// create array to store info
	final String[] result = new String[info.length];
	for (int i = 0; i < info.length; i++)
	   {
	   result[i] = info[i].getName();
	   }

	return result;
	}

   /**
    * Returns information about all installed Look & Feels in a sorted array.
    * @return
    */
   public static LookAndFeelInfo[] getInstalledLookAndFeels()
	{
	// get all installed l&f and sort them
	final UIManager.LookAndFeelInfo[] info = UIManager.getInstalledLookAndFeels();

	// copy into new format
	final LookAndFeelInfo[] result = new LookAndFeelInfo[info.length];
	for (int i = 0; i < info.length; ++i)
	   {
	   result[i] = new LookAndFeelInfo(info[i]);
	   }

	// sort them
	Arrays.sort(result);

	// return themn
	return result;
	}

   public String getLookAndFeel()
	{
	final UIManager.LookAndFeelInfo selected = (UIManager.LookAndFeelInfo) getSelectedValue();
	return selected.getClassName();
	}

   protected int getReturnValue()
	{
	int result = super.getReturnValue();

	if (result == OK && set != null)
	   {
	   try
		{
		UIManager.setLookAndFeel(getLookAndFeel());
		SwingUtilities.updateComponentTreeUI(SwingUtilities.getRoot(set));
		}
	   catch (UnsupportedLookAndFeelException un)
		{
		showError(JReqTools.getResourceString("Error.01", getLookAndFeel()));
		}
	   catch (Exception x)
		{
		System.err.println(x);
		}
	   }

	return result;
	}

   public void setAfterSelection(Component tree)
	{
	set = tree;
	}
   }
