/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * This is a progress bar which pops up in an own dialog.
 *
 * @author	Markus Hillenbrand
 */
public class ProgressTracker extends AbstractRequester
   {
   private ProgressTask task = null;
   private boolean done = false;
   private boolean cancelButton = false;
   private JFrame parentFrame = null;
   private JInternalFrame parentJFrame;

   private class ProgressTrackerDialog extends HDialog
	{
	private JProgressBar progressbar = new JProgressBar(JProgressBar.HORIZONTAL);
	private JLabel text = new JLabel("-");
	private Timer timer = null;
	private HMultiLineLabel message = new HMultiLineLabel(ProgressTracker.this.getMessage(), HMultiLineLabel.LEFT);
	private JButton cancel = new JButton(JReqTools.getResourceString("Cancel"), UIManager.getIcon(UIPreferences.BUTTON_ICON_CANCEL));

	public ProgressTrackerDialog(JDialog parent)
	   {
	   super(parent, "JReqTools: ProgressTracker");
	   init();
	   }

	public ProgressTrackerDialog(JFrame parent)
	   {
	   super(parent, "JReqTools: ProgressTracker");
	   init();
	   }

	private void init()
	   {
	   // init components
	   progressbar.setStringPainted(true);
	   message.setFont(UIPreferences.getMessageFont());

	   // listeners
	   cancel.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   cancel.setEnabled(false);
		   task.cancel();
		   }
		});

	   // get the constraints
	   HGridBagConstraints c = UIPreferences.getGridBagConstraints();

	   // create icon label
	   final JLabel label = new JLabel(UIManager.getIcon(UIPreferences.DIALOG_ICON_WORKING));
	   label.setHorizontalAlignment(SwingConstants.LEFT);
	   label.setVerticalAlignment(SwingConstants.TOP);

	   // create textfield/choice panel
	   final JPanel pbpanel = new JPanel(new GridBagLayout());
	   c.set(1, 0, 2, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   pbpanel.add(new HSeparator(), c);
	   c.set(0, 1, 1, 1, 0, 0, GridBagConstraints.NONE);
	   pbpanel.add(label, c);
	   c.set(1, 1, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   pbpanel.add(message, c);
	   c.set(1, 2, 2, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   pbpanel.add(new HSeparator(), c);
	   c.set(1, 3, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   pbpanel.add(progressbar, c);

	   // create content panel
	   final JPanel panel = new JPanel(new GridBagLayout());
	   panel.setBorder(UIPreferences.getFrameBorder());

	   // add components
	   c.insets = new Insets(0, 0, 0, 0);
	   c.set(0, 0, 2, 1, 1, 1, GridBagConstraints.BOTH);
	   panel.add(pbpanel, c);
	   c = UIPreferences.getGridBagConstraints();
	   c.set(0, 1, 2, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   panel.add(new HSeparator(), c);
	   c.set(0, 2, cancelButton ? 1 : 2, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   panel.add(text, c);
	   if (cancelButton)
		{
		c.set(1, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		panel.add(cancel, c);
		}

	   // finish
	   setContentPane(panel);
	   pack();
	   if (getWidth() < 400)
		{
		setSize(400, getHeight());
		}
	   }

	public void setVisible(boolean b)
	   {
	   if (!b)
		{
		// Timer has already been stopped();
		super.setVisible(false);
		return;
		}

	   progressbar.setMinimum(task.getMinimum());
	   progressbar.setMaximum(task.getMaximum());
	   progressbar.setValue(task.getCurrent());

	   final ActionListener l = new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   if (done)
			{
			timer.stop();
			timer = null;
			setVisible(false);
			}
		   else
			{
			// it is safe to call swing methods because the Timer is being executed in the
			// rendering thread, see javax.swing.Timer documentation for more information
			text.setText(task.getCurrentText());
			progressbar.setMinimum(task.getMinimum());
			progressbar.setMaximum(task.getMaximum());
			progressbar.setValue(task.getCurrent());
			}
		   }
		};

	   timer = new Timer(1000, l);
	   timer.start();

	   super.setVisible(true);
	   }
	}

   private class UserThread extends Thread
	{
	public UserThread()
	   {
	   setName("JavaGPE-ProgressTracker");
	   setPriority(MIN_PRIORITY);
	   }

	public void run()
	   {
	   try
		{
		sleep(100);
		}
	   catch (InterruptedException x)
		{
		}

	   done = false;

	   try
		{
		task.run();
		}
	   catch (Exception x)
		{
		System.err.println(x);
		}

	   done = true;
	   }
	}

   public ProgressTracker(String message, ProgressTask task)
	{
	this("JReqTools: ProgressTracker", message, task);
	}

   public ProgressTracker(String title, String message, ProgressTask task)
	{
	super(title, message);

	this.task = task;
	}

   public void addCancelButton(boolean bool)
	{
	cancelButton = bool;
	}

   protected JPanel createContentPane()
	{
	return new JPanel();
	}

   protected int getReturnValue()
	{
	return OK;
	}

   protected void prepareForRequest()
	{
	dialog = new ProgressTrackerDialog(parentFrame);
	task.setDialog(dialog);
	}

   public void start(JComponent parentComponent)
	{
	parentJFrame = SwingUtils.getJInternalFrame(parentComponent);
	parentFrame = parentJFrame != null ? null : SwingUtils.getJFrame(parentComponent);
	final Thread t = new UserThread();
	t.start();
	request(parentFrame);
	}

   public void start(JFrame parent)
	{
	parentFrame = parent;
	parentJFrame = null;
	final Thread t = new UserThread();
	t.start();
	request(parentFrame);
	}

   public void start(JInternalFrame parent)
	{
	parentFrame = null;
	parentJFrame = parent;
	final Thread t = new UserThread();
	t.start();
	request(parentFrame);
	}
   }
