/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import java.awt.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.io.*;
import java.util.*;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JScrollPane;
import javax.swing.text.*;

/**
 * A Requester that loads a file, and display is for editing.
 */
public class PropertyFileEditorRequester extends OkCancelRequester
   {
   // components
   protected final HLabel label = new HLabel("", HLabel.LEFT);
   protected final Hashtable<String, JTextComponent> components = new Hashtable<String, JTextComponent>();
   // Property: file
   protected File file = null;
   protected PropertyEditorElement[] elements;

   /**
    * Constructs a new FileEditorRequester with a default title and message, and a given file.
    * @param file
    */
   public PropertyFileEditorRequester(File file, PropertyEditorElement[] elements)
	{
	this("JReqTools: PropertiesFileEditorRequester", null, file, elements);
	}

   /**
    * Constructs a new FileEditorRequester with a default title, a given message, and file.
    * @param message
    * @param file
    */
   public PropertyFileEditorRequester(String message, File file, PropertyEditorElement[] elements)
	{
	this("JReqTools: PropertiesFileEditorRequester", message, file, elements);
	}

   /**
    * Constructs a new FileEditorRequester with a given title, message, and file.
    * @param title
    * @param message
    * @param file
    */
   public PropertyFileEditorRequester(String title, String message, File file, PropertyEditorElement[] elements)
	{
	super(title, message);

	// init
	this.elements = elements;
	this.file = file;
	this.label.setText(file != null ? file.getAbsolutePath() : "ERROR: No file associated.");

	// init components
	label.setFont(UIPreferences.getAcceleratorFont());
	label.setForeground(Color.gray);
	}

   /**
    * Returns the value of the file property.
    * @return
    */
   public File getFile()
	{
	return file;
	}

   protected HPanel getUserPanel()
	{
	// create panel
	final HPanel panel = new HPanel(new GridBagLayout());
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();
	c.set(0, 0, 1, 1, 1, 1, GridBagConstraints.BOTH);
	panel.add(new HScrollPane(createComponents()), c);
	c.set(0, 1, 1, 0, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(label, c);
	return panel;
	}

   @Override
   protected void ok()
	{
	try
	   {
	   final Properties props = new Properties();
	   props.load(new FileInputStream(file));

	   for (String key : components.keySet())
		{
		final JTextComponent current = components.get(key);
		final String text = current instanceof JFormattedTextField
			  ? ((JFormattedTextField) current).getValue().toString()
			  : current.getText();
		props.setProperty(key, text);
		}

	   props.store(new FileOutputStream(file), "Edited with the Venice Service Life-cycle Manager.");
	   }
	catch (Exception x)
	   {
	   label.setText(x.getLocalizedMessage());
	   }
	}

   @Override
   protected void prepareForRequest()
	{
	try
	   {
	   final Properties props = new Properties();
	   props.load(new FileInputStream(file));
	   for (String key : components.keySet())
		{
		final JTextComponent current = components.get(key);
		final String value = props.getProperty(key, ""); // TODO: use default value
		current.setText(value);
		}
	   }
	catch (Exception x)
	   {
	   label.setText(x.getLocalizedMessage());
	   }
	}

   /**
    * Assigns the value of the file property.
    * @param f 
    */
   public void setFile(File f)
	{
	this.file = f;
	this.label.setText(file != null ? file.getAbsolutePath() : "ERROR: No file associated.");
	}

   private HPanel createComponents()
	{
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();
	final HPanel result = new HPanel(new GridBagLayout());
	result.setBorder(UIPreferences.getInnerTabbedPaneBorder());

	int y = 0;
	for (PropertyEditorElement e : elements)
	   {
	   final JTextComponent component = e.getType().getJTextComponent(e.getDefaultValue());
	   components.put(e.getKey(), component);

	   c.set(0, y, 1, 1, 0, 0, HGridBagConstraints.HORIZONTAL, HGridBagConstraints.DEFAULT_VERTICAL_INSETS);
	   result.add(new HLabel(e.getDescription() + ":", HLabel.LEFT), c);
	   c.set(1, y, 1, 1, 1, e.getType().getWeightY(), HGridBagConstraints.BOTH, HGridBagConstraints.DEFAULT_VERTICAL_INSETS);
	   result.add(e.getType().needsScrollPane() ? new JScrollPane(component) : component, c);

	   final JComponent additional = e.getType().getAdditionalComponent(component);
	   if (additional != null)
		{
		c.set(2, y, 1, 1, 0, 0, HGridBagConstraints.BOTH, HGridBagConstraints.DEFAULT_VERTICAL_INSETS);
		result.add(additional, c);
		}

	   y++;
	   }

	return result;
	}
   }
