/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.HGridBagConstraints;
import de.hillenbrand.system.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;

/**
 * This is a requester for a list of Objects. The user can select one of them.
 *
 * @param <E>
 * @author	Markus Hillenbrand
 */
public class SelectRequester<E> extends OkCancelRequester
   {
   private Object[] objects = null;
   private Object[] selected = new Object[0];
   // components
   private JList list = new JList();
   private JScrollPane scrollpane = new JScrollPane(list, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
   // Property: multiSelection
   protected boolean multiSelection = false;
   // Property: comparator
   protected Comparator comparator = null;

   /**
    * Constructs a new SelectRequester without a message text.
    *
    * @param	data - The data to select from.
    */
   public SelectRequester(Object[] data)
	{
	this("JReqTools: SelectRequester", null, data, null);
	}

   /**
    * Constructs a new SelectRequester.
    *
    * @param	message - The message to be shown in the requester.
    * @param	data - The data to select from.
    */
   public SelectRequester(String message, Object[] data)
	{
	this("JReqTools: SelectRequester", message, data, null);
	}

   /**
    * Constructs a new SelectRequester.
    *
    * @param	message - The message to be shown in the requester.
    * @param	data - The data to select from.
    * @param	selected - The data object selected by default.
    */
   public SelectRequester(String message, Object[] data, Object selected)
	{
	this("JReqTools: SelectRequester", message, data, selected);
	}

   /**
    * Constructs a new SelectRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	data - The data to select from.
    */
   public SelectRequester(String title, String message, Object[] data)
	{
	this(title, message, data, null);
	}

   /**
    * Constructs a new SelectRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	data - The data to select from.
    * @param	selected - The data object selected by default.
    */
   public SelectRequester(String title, String message, Object[] data, Object selected)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_SELECTDATA));

	// init variables
	setMultiSelection(false);
	setValues(data);
	setSelectedValue(selected);

	// init components
	list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	scrollpane.setPreferredSize(new java.awt.Dimension(200, 200));
	}

   /**
    * Constructs a new SelectRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	data - The data to select from.
    * @param	selected - The data objects selected by default.
    */
   public SelectRequester(String title, String message, Object[] data, Object[] selected)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_SELECTDATA));

	// init variables
	setMultiSelection(true);
	setValues(data);
	setSelectedValues(selected);

	// init components
	list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
	scrollpane.setPreferredSize(new java.awt.Dimension(200, 200));
	}

   /**
    * Returns the value of the comparator property.
    * @return
    */
   public Comparator getComparator()
	{
	return comparator;
	}

   protected JComponent getOkTriggerComponent()
	{
	return list;
	}

   public Object getSelectedValue()
	{
	return selected == null || selected.length == 0 ? null : selected[0];
	}

   public Object[] getSelectedValues()
	{
	return selected;
	}

   protected JPanel getUserPanel()
	{
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();
	final JPanel panel = new JPanel(new GridBagLayout());
	c.set(0, 0, 1, 1, 1, 1, GridBagConstraints.BOTH);
	panel.add(scrollpane, c);
	return panel;
	}

   /**
    * Returns the value of the multiSelection property.
    * @return
    */
   public boolean isMultiSelection()
	{
	return multiSelection;
	}

   protected void ok()
	{
	selected = list.getSelectedValues();
	}

   @SuppressWarnings("unchecked")
   protected void prepareForRequest()
	{
	list.setSelectionMode(multiSelection ? ListSelectionModel.MULTIPLE_INTERVAL_SELECTION : ListSelectionModel.SINGLE_SELECTION);
	list.setListData(objects);

	if (isMultiSelection())
	   {
	   final ListSelectionModel model = list.getSelectionModel();
	   for (int i = 0; i < selected.length; ++i)
		{
		final int index = ArrayUtils.indexOf(objects, selected[i], comparator);
		if (index == -1)
		   {
		   continue;
		   }
		model.addSelectionInterval(index, index);
		}
	   }
	else
	   {
	   if (selected != null && selected.length > 0)
		{
		list.setSelectedValue(selected[0], true);
		}
	   if (objects != null && list.getSelectedValue() == null)
		{
		list.setSelectedIndex(0);
		}
	   }
	}

   /**
    * Assigns the value of the comparator property.
    * @param arg
    */
   public void setComparator(Comparator arg)
	{
	comparator = arg;
	}

   public void setListCellRenderer(ListCellRenderer lcr)
	{
	list.setCellRenderer(lcr);
	}

   /**
    * Assigns the value of the multiSelection property.
    * @param arg
    */
   public void setMultiSelection(boolean arg)
	{
	multiSelection = arg;
	}

   public void setSelectedValue(Object object)
	{
	selected = object == null ? new Object[0] : ArrayUtils.toArray(object);
	}

   public void setSelectedValues(Object[] objects)
	{
	selected = objects == null ? new Object[0] : objects;
	}

   /**
    * Sets the string in the requester.
    * @param data
    */
   public void setValues(Object[] data)
	{
	objects = data;
	}
   }
