/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.jreqtools;


/**
* Some string checkers for StringRequester.
* 
* @author	Markus Hillenbrand
*/

public class StringCheckers {
	private static final boolean DEBUG = false;

	
	/**
	* Returns the checker for Integer values.
	*/

	public static StringChecker getByteChecker()  {
		return new StringChecker() 
			{
			public boolean accept(String s)
				{
				if (s.length() == 0) return false;
				try { Byte.valueOf(s); } catch (NumberFormatException x) { return false; }
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.ByteChecker.FailMessage"); }
			};
	}

	/**
	* Returns the checker for Double values.
	*/

	public static StringChecker getDoubleChecker()  {
		return new StringChecker() 
			{
			public boolean accept(String s)
				{
				if (s.length() == 0) return false;
				try { Double.valueOf(s); } catch (NumberFormatException x) { return false; }
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.DoubleChecker.FailMessage"); }
			};
	}

	/**
	* Returns the checker for Float values.
	*/

	public static StringChecker getFloatChecker()  {
		return  new StringChecker() 
			{
			public boolean accept(String s)
				{
				if (s.length() == 0) return false;
				try { Float.valueOf(s); } catch (NumberFormatException x) { return false; }
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.FloatChecker.FailMessage"); }
			};
	}

	/**
	* Returns the checker for Integer values.
	*/

	public static StringChecker getIntegerChecker()  {
		return new StringChecker() 
			{
			public boolean accept(String s)
				{
				if (s.length() == 0) return false;
				try { Integer.parseInt(s); } catch (NumberFormatException x) { return false; }
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.IntegerChecker.FailMessage"); }
			};
	}

	/**
	* Returns the checker for a string which can be used as a Java identifier.
	*/

	public static StringChecker getJavaIdentifierChecker()  {
		return new StringChecker() 
			{
			public boolean accept(String s)
				{
				int len = s.length();
				if (len == 0) return false;
				if (! Character.isJavaIdentifierStart(s.charAt(0))) return false;
				for (int i=1; i<len; ++i) if (! Character.isJavaIdentifierPart(s.charAt(i))) return false;
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.JavaIdentifierChecker.FailMessage"); }
			};
	}

	/**
	* Returns the checker for a string which can be used as a Java package identifier.
	*/

	public static StringChecker getJavaPackageChecker()  {
		return new StringChecker() 
			{
			public boolean accept(String s)
				{
				int len = s.length();
				if (len == 0) return false;
				if (s.endsWith(".")) return false;
				if (! Character.isJavaIdentifierStart(s.charAt(0))) return false;
				for (int i=1; i<len; ++i) 
					{
					char c = s.charAt(i);
					if (c == '.') continue;
					if (! Character.isJavaIdentifierPart(s.charAt(i)) ) return false;
					}
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.JavaPackageChecker.FailMessage"); }
			};
	}

	/**
	* Returns the checker for Integer values.
	*/

	public static StringChecker getLongChecker()  {
		return new StringChecker() 
			{
			public boolean accept(String s)
				{
				if (s.length() == 0) return false;
				try { Long.parseLong(s); } catch (NumberFormatException x) { return false; }
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.LongChecker.FailMessage"); }
			};
	}

	/**
	* Returns the checker for Integer values.
	*/

	public static StringChecker getShortChecker()  {
		return new StringChecker() 
			{
			public boolean accept(String s)
				{
				if (s.length() == 0) return false;
				try { Short.valueOf(s); } catch (NumberFormatException x) { return false; }
				return true;
				}
			public String getFailMessage() { return JReqTools.getResourceString("StringCheckers.ShortChecker.FailMessage"); }
			};
	}

}
