/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import javax.swing.*;

/**
 * A requester for simple strings.
 *
 * @author	Markus Hillenbrand
 */
public class StringRequester extends OkCancelRequester
   {
   private Object[] choices = null;
   private Object selected = null;
   private String string = "";
   private StringChecker checker = null;
   // components
   private final HTextField textfield = new HTextField();
   private final HComboBox<String> combobox = new HComboBox<String>("");

   /**
    * Constructs a new StringRequester with an empty default value.
    *
    * @param	message - The message to be shown in the requester.
    * @param	defaultstring - The initial value of the text field.
    */
   public StringRequester(String message, String defaultstring)
	{
	this("JReqTools: StringRequester", message, defaultstring);

	combobox.setEditable(true);
	}

   /**
    * Constructs a new StringRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	defaultstring - The initial value of the text field.
    */
   public StringRequester(String title, String message, String defaultstring)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ENTERDATA));

	combobox.setEditable(true);
	setString(defaultstring);
	}

   /**
    * Returns the component that will trigger the Ok button.
    * @return
    */
   protected JComponent getOkTriggerComponent()
	{
	return choices != null ? (JComponent) combobox : (JComponent) textfield;
	}

   /**
    * Gets the string that has been entered or selected by the user.
    * @return
    */
   public String getString()
	{
	return string;
	}

   /**
    * Returns the panel of the requester.
    * @return
    */
   protected JPanel getUserPanel()
	{
	// get default constraints
	final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	// create panel
	final JPanel panel = new JPanel(new GridBagLayout());
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(choices != null ? (JComponent) combobox : (JComponent) textfield, c);

	return panel;
	}

   /**
    * This method will be called when the user presses the Ok button.
    */
   protected void ok()
	{
	string = choices != null ? combobox.getSelectedItem().toString() : textfield.getText();
	}

   /**
    * This method will be called right before the request will be shown.
    */
   protected void prepareForRequest()
	{
	if (choices != null)
	   {
	   combobox.setModel(new DefaultComboBoxModel(choices));
	   if (selected != null)
		{
		combobox.setSelectedItem(selected);
		}
	   else
		{
		combobox.setSelectedItem(string);
		}
	   combobox.requestFocus();
	   combobox.getEditor().selectAll();
	   }
	else
	   {
	   textfield.setText(string);
	   }
	}

   /**
    * The values of the array will be presented to the user as default values.
    *
    * @param	choices The possible values.
    */
   public void setDefaultChoices(Object[] choices)
	{
	this.choices = choices;
	this.selected = null;
	}

   /**
    * The values of the array will be presented to the user as default values.
    *
    * @param	choices The possible values.
    * @param	selected The initially selected default value. This will overwrite the value set using setString()
    */
   public void setDefaultChoices(Object[] choices, Object selected)
	{
	this.choices = choices;
	this.selected = selected;
	}

   /**
    * Sets the string in the requester.
    * @param newstring
    */
   public void setString(String newstring)
	{
	string = newstring;
	}

   /**
    * Sets a StringChecker to be used for accepting an entered string.
    *
    * @param	newchecker The string checker that will be used for testing the values entered by the user.
    */
   public void setStringChecker(StringChecker newchecker)
	{
	checker = newchecker;
	}

   /**
    * This method is used to validate the data entered by the user.
    *
    * @return error string, null if everything is ok
    */
   public String validateData()
	{
	final String value = choices != null ? combobox.getSelectedItem().toString() : textfield.getText();

	if (checker != null && !checker.accept(value))
	   {
	   return checker.getFailMessage();
	   }
	else
	   {
	   return null;
	   }
	}
   }
