/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.system.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/**
 * This component implements a collection of a text field an a button, with which you can select a file or a directory.
 */
public class HFileString extends Box
   {
   // policy constants
   public static final int FILES_AND_DIRECTORIES = 0;
   public static final int DIRECTORIES_ONLY = 1;
   public static final int MULTIPLE_FILES = 2;
   // variables
   private int policy = FILES_AND_DIRECTORIES;
   private final Vector<ActionListener> listeners = new Vector<ActionListener>();
   private final FileRequester fileRequester = new FileRequester();
   // components
   private final HTextField textfield = new HTextField();
   private final JButton button = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_OPEN));
   private final JButton clear = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_CLEAR));

   /**
    * Constructs a new FileString.
    * The component's policy is FILES_AND_DIRECTORIES.
    */
   public HFileString()
	{
	this(new File(System.getProperty("user.home")), FILES_AND_DIRECTORIES);
	}

   /**
    * Constructs a new FileString with the file as content.
    * The component's policy is FILES_AND_DIRECTORIES.
    *
    * @param	file the initial file
    */
   public HFileString(File file)
	{
	this(file.toString(), FILES_AND_DIRECTORIES);
	}

   /**
    * Constructs a new FileString with the file as content.
    * The component's policy is FILES_AND_DIRECTORIES.
    *
    * @param	filename the initial file name
    */
   public HFileString(String filename)
	{
	this(filename, FILES_AND_DIRECTORIES);
	}

   /**
    * Constructs a new FileString the policy.
    *
    * @param	policy either FILES_AND_DIRECTORIES or DIRECTORIES_ONLY or MULTIPLE_FILES
    */
   public HFileString(int policy)
	{
	this("", policy);
	}

   /**
    * Constructs a new FileString with filename and the policy.
    *
    * @param	file the file
    * @param	policy either FILES_AND_DIRECTORIES or DIRECTORIES_ONLY or MULTIPLE_FILES
    */
   public HFileString(File file, int policy)
	{
	this(file.toString(), policy);
	}

   /**
    * Constructs a new FileString with the file as content.
    * The component's policy is FILES_AND_DIRECTORIES.
    *
    * @param	filename the initial file name
    * @param	policy the file requester policy
    */
   public HFileString(String filename, int policy)
	{
	super(BoxLayout.X_AXIS);

	// init variables
	this.policy = policy;

	// init components
	if (policy == MULTIPLE_FILES)
	   {
	   textfield.setEnabled(false);
	   textfield.setText(Helper.GRS("HFileString.MultipleFiles"));
	   }
	else
	   {
	   textfield.setText(filename);
	   }
	button.setMargin(new java.awt.Insets(0, 0, 0, 0));
	button.setRequestFocusEnabled(false);
	clear.setMargin(new java.awt.Insets(0, 0, 0, 0));
	clear.setRequestFocusEnabled(false);
	fileRequester.setTitle(Helper.GRS("HFileString.Title"));
	fileRequester.setDirectoriesOnly(policy == DIRECTORIES_ONLY);
	fileRequester.setMultiSelection(policy == MULTIPLE_FILES);

	// listeners
	textfield.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		textEntered();
		}
	   });
	button.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		request();
		}
	   });
	clear.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		fileRequester.setFiles(new File[0]);
		textfield.setText(Helper.GRS("HFileString.NoOfFiles", 0));
		}
	   });

	// create box
	add(textfield);
	if (policy == MULTIPLE_FILES)
	   {
	   add(clear);
	   }
	add(button);
	}

   /**
    * Get the policy of the FileString component.
    * @return
    */
   public int getPolicy()
	{
	return policy;
	}

   /**
    * Adds an action listener to this components.
    *
    * @param	l the listener
    */
   public void addActionListener(ActionListener l)
	{
	listeners.addElement(l);
	}

   public void addChoosableFileFilter(javax.swing.filechooser.FileFilter f)
	{
	fileRequester.addChoosableFileFilter(f);
	}

   /**
    * Returns the current value of the component.
    *
    * @return	the file which has been selected by the user (the file or directory might not exist)
    */
   public File getFile()
	{
	if (policy == MULTIPLE_FILES)
	   {
	   throw new RuntimeException("Please use getFiles() because HFileString is in multi-file mode.");
	   }

	final String s = textfield.getText();
	return s.length() == 0 ? null : new File(s);
	}

   public FileRequester getFileRequester()
	{
	return fileRequester;
	}

   /**
    * Returns the current value of the component.
    *
    * @return	the files which has been selected by the user
    */
   public File[] getFiles()
	{
	if (policy != MULTIPLE_FILES)
	   {
	   throw new RuntimeException("Please use getFile() because HFileString is in single-file mode.");
	   }
	return fileRequester.getFiles();
	}

   /**
    * Returns the name of the selected file.
    *
    * @return	the name of the file which has been selected by the user (the file or directory might not exist)
    */
   public String getText()
	{
	return textfield.getText();
	}

   void request()
	{
	String s = textfield.getText();
	if (s.length() == 0)
	   {
	   s = System.getProperty("user.home");
	   }
	File file = new File(s);

	if (policy == DIRECTORIES_ONLY)
	   {
	   if (file.isFile())
		{
		file = new File(file.getParent());
		}
	   if (!file.exists())
		{
		if (JReqTools.showQuestion(new JFrame(), Helper.GRS("HFileString.CreateDirectory", file.toString())))
		   {
		   file.mkdirs();
		   }
		}
	   fileRequester.setFile(file, "");
	   }
	else if (policy == FILES_AND_DIRECTORIES)
	   {
	   final File dir = file.isDirectory() ? file : new File(file.getParent() == null ? System.getProperty("user.home") : file.getParent());
	   final String filename = file.isDirectory() ? "" : file.getName();
	   fileRequester.setFile(dir, filename);
	   }

	if (fileRequester.request(SwingUtils.getJFrame(this)) != FileRequester.OK)
	   {
	   return;
	   }

	if (policy == MULTIPLE_FILES)
	   {
	   final File[] selected = fileRequester.getFiles();
	   textfield.setText(Helper.GRS("HFileString.NoOfFiles", selected.length));
	   }
	else
	   {
	   textfield.setText(fileRequester.getFile().toString());
	   textEntered();
	   }
	}

   public void setEditable(boolean b)
	{
	textfield.setEditable(b);
	}

   public void setEnabled(boolean b)
	{
	textfield.setEnabled(b);
	button.setEnabled(b);
	}

   /**
    * Sets the file.
    *
    * @param	file the new selected file of the component
    */
   public void setFile(File file)
	{
	textfield.setText(file.toString());
	}

   /**
    * Sets the file.
    *
    * @param	file the new selected file of the component
    */
   public void setFile(String file)
	{
	textfield.setText(file);
	}

   public void setLoadMode()
	{
	fileRequester.setLoadMode();
	}

   public void setSaveMode()
	{
	fileRequester.setSaveMode();
	}

   /**
    * Sets the text.
    *
    * @param	file the new selected file of the component
    */
   public void setText(String file)
	{
	textfield.setText(file);
	}

   /**
    * Sets the tool tip for this component.
    *
    * @param	text the tool tip text
    */
   public void setToolTipText(String text)
	{
	textfield.setToolTipText(text);
	button.setToolTipText(text);
	}

   void textEntered()
	{
	// Check for correct suffix
	final javax.swing.filechooser.FileFilter filter = fileRequester.getSelectedFileFilter();
	if (filter != null && filter instanceof de.hillenbrand.jreqtools.FileFilter.SuffixFileFilter)
	   {
	   String suffix = ((de.hillenbrand.jreqtools.FileFilter.SuffixFileFilter) filter).getSuffix();
	   String text = textfield.getText();
	   if (!text.endsWith(suffix))
		{
		text += suffix;
		textfield.setText(text);
		}
	   }

	// process listeners
	final ActionEvent ev = new ActionEvent(this, 0, "");
	for (Enumeration e = listeners.elements(); e.hasMoreElements();)
	   {
	   ((ActionListener) e.nextElement()).actionPerformed(ev);
	   }
	}
   }
