/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.swing.text.*;
import de.hillenbrand.system.*;
import java.awt.event.*;
import java.text.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * HTextField is a lightweight component that allows the editing of a single line of text, similar to JFormattedTextField.
 * But it additionally has an undo / redo buffer and a cut & paste menu.
 *
 * @param <T>
 * @see javax.swing.JFormattedTextField
 */
public class HFormattedTextField<T> extends JFormattedTextField
   {
   private JPopupMenu popupmenu = null;
   private MouseListener mouselistener = null;

   /**
    * Creates a HFormattedTextField with no AbstractFormatterFactory. Use setMask or setFormatterFactory to configure the JFormattedTextField to edit a particular type of value.
    */
   public HFormattedTextField()
	{
	super();
	init();
	}

   /**
    * Creates a HFormattedTextField. format is wrapped in an appropriate AbstractFormatter which is then wrapped in an AbstractFormatterFactory.
    * @param f
    */
   public HFormattedTextField(Format f)
	{
	super(f);
	init();
	}

   /**
    * Creates a HFormattedTextField with the specified AbstractFormatter. The AbstractFormatter is placed in an AbstractFormatterFactory.
    * @param f
    */
   public HFormattedTextField(JFormattedTextField.AbstractFormatter f)
	{
	super(f);
	init();
	}

   /**
    * Creates a HFormattedTextField with the specified AbstractFormatterFactory.
    * @param f
    */
   public HFormattedTextField(JFormattedTextField.AbstractFormatterFactory f)
	{
	super(f);
	init();
	}

   /**
    * Creates a HFormattedTextField with the specified value. This will create an AbstractFormatterFactory based on the type of value.
    * @param value
    */
   public HFormattedTextField(T value)
	{
	super(value);
	init();
	// setValue() must be called again as init() changes the underlying document
	setValue(value);
	}

   /**
    * Creates a HFormattedTextField with the specified AbstractFormatterFactory and initial value.
    * @param f 
    * @param currentValue
    */
   public HFormattedTextField(JFormattedTextField.AbstractFormatterFactory f, T currentValue)
	{
	super(f);
	init();
	}

   public Action getAction(String name)
	{
	final Action[] actions = getActions();
	for (int i = 0; i < actions.length; ++i)
	   {
	   if (actions[i].getValue(Action.NAME).equals(name))
		{
		return actions[i];
		}
	   }

	return null;
	}

   public JPopupMenu getJPopupMenu()
	{
	return popupmenu;
	}

   public KeyStroke getKeyStrokeForAction(String action)
	{
	// get input map
	final InputMap im = (InputMap) UIManager.get("TextField.focusInputMap");
	if (im == null)
	   {
	   return null;
	   }

	// create result
	KeyStroke result1 = null;
	KeyStroke result2 = null;

	// find result
	final KeyStroke[] keys = im.allKeys();
	for (int i = 0; i < keys.length; ++i)
	   {
	   final String s = (String) im.get(keys[i]);
	   if (result1 == null && s != null && s.equals(action))
		{
		result1 = keys[i];
		}
	   else if (result2 == null && s != null && s.equals(action))
		{
		result2 = keys[i];
		}
	   }

	// results prüfen
	if (result1 != null && result2 != null && result1.getModifiers() == 0)
	   {
	   return result2;
	   }
	return result1;
	}

   protected void _colorize()
	{
	HFormattedTextField.this.setBackground(isEditValid() ? UIManager.getColor("TextField.background") : java.awt.Color.red);
	//HFormattedTextField.this.setBackground(isEditValid() ? Color.green : Color.red);
	}

   private void init()
	{
	// set document
	this.setDocument(new UndoableDocument());
	// undo
	this.addKeyListener(new KeyAdapter()
	   {
	   public void keyReleased(KeyEvent e)
		{
		// System.out.println(getText() + " -> " + isEditValid());
		_colorize();
		}
	   public void keyPressed(KeyEvent e)
		{
		if (e.getKeyCode() == KeyEvent.VK_Z && e.getModifiersEx() == KeyEvent.CTRL_DOWN_MASK)
		   {
		   undo();
		   }
		else if (e.getKeyCode() == KeyEvent.VK_Z && e.getModifiersEx() == (KeyEvent.CTRL_DOWN_MASK | KeyEvent.SHIFT_DOWN_MASK))
		   {
		   redo();
		   }
		}
	   });

	// menu
	popupmenu = new JPopupMenu(Helper.GRS("HTextField.PopupMenu.Title"));

	// mouse listener
	mouselistener = SwingUtils.createMouseListenerForPopupMenu(this, popupmenu);
	this.addMouseListener(mouselistener);

	// items
	JMenuItem item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Undo"), UIManager.getIcon(UIPreferences.MENU_ICON_UNDO)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		undo();
		}
	   });
	item.setAccelerator(KeyStroke.getKeyStroke("control Z"));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Redo"), UIManager.getIcon(UIPreferences.MENU_ICON_REDO)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		redo();
		}
	   });
	item.setAccelerator(KeyStroke.getKeyStroke("control shift Z"));

	popupmenu.addSeparator();
	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Cut"), UIManager.getIcon(UIPreferences.MENU_ICON_CUT)));
	item.addActionListener(getAction(DefaultEditorKit.cutAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.cutAction));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Copy"), UIManager.getIcon(UIPreferences.MENU_ICON_COPY)));
	item.addActionListener(getAction(DefaultEditorKit.copyAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.copyAction));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Paste"), UIManager.getIcon(UIPreferences.MENU_ICON_PASTE)));
	item.addActionListener(getAction(DefaultEditorKit.pasteAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.pasteAction));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Delete"), UIManager.getIcon(UIPreferences.MENU_ICON_DELETE)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		setText("");
		}
	   });

	popupmenu.addSeparator();
	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.SelectAll"), UIManager.getIcon(UIPreferences.MENU_ICON_SELECTALL)));
	item.addActionListener(getAction(DefaultEditorKit.selectAllAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.selectAllAction));

	_colorize();
	}

   public void redo()
	{
	try
	   {
	   final UndoableDocument document = (UndoableDocument) this.getDocument();
	   if (document.canRedo())
		{
		document.redo();
		}
	   else
		{
		SwingUtils.beep();
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }
	}

   public void setEditable(boolean b)
	{
	if (b)
	   {
	   if (mouselistener != null)
		{
		addMouseListener(mouselistener);
		}
	   }
	else
	   {
	   if (mouselistener != null)
		{
		removeMouseListener(mouselistener);
		}
	   }

	super.setEditable(b);
	}

   public void setEnabled(boolean b)
	{
	if (b)
	   {
	   if (mouselistener != null)
		{
		addMouseListener(mouselistener);
		}
	   }
	else
	   {
	   if (mouselistener != null)
		{
		removeMouseListener(mouselistener);
		}
	   }

	super.setEnabled(b);
	}

   public void undo()
	{
	try
	   {
	   final UndoableDocument document = (UndoableDocument) this.getDocument();
	   if (document.canUndo())
		{
		document.undo();
		}
	   else
		{
		SwingUtils.beep();
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }
	}

   @Override
   @SuppressWarnings("unchecked")
   public T getValue()
	{
	return (T) super.getValue();
	}

   }
