/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing;

import java.awt.*;
import java.awt.image.*;
import javax.swing.JPanel;

/**
* Simple panel subclass which renders a 2-color vertical linear gradient
* as the background.
*/

public class HGradientPanel extends JPanel {
	private static final boolean DEBUG = false;

	private final Color[] colors = new Color[2];
	private Image image;
	
	public HGradientPanel(Color color1, Color color2)  {
		setOpaque(false);  // required to disable automatic bg painting
		setBackground(color1); // in case colors are derived from background
		
		colors[0] = color1;
		colors[1] = color2;
	}

	protected void changeGradientColor(int colorIndex, Color newColor)  {
		Color oldColor = colors[colorIndex];
		colors[colorIndex] = newColor;
		if (! oldColor.equals(newColor))
			{
			image = null;
			firePropertyChange("gradientColor"+colorIndex, oldColor, newColor);
			}
	}

	protected BufferedImage createCompatibleImage(int width, int height)  {
		return GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage(width, height);
		                   
	}

	protected BufferedImage createGradientImage(int width, int height, Color gradient1, Color gradient2)  {
		final BufferedImage gradientImage = createCompatibleImage(width, height);
		final GradientPaint gradient = new GradientPaint(0, 0, gradient1, 0, height, gradient2, false);
		final Graphics2D g2 = (Graphics2D)gradientImage.getGraphics();
		
		g2.setPaint(gradient);
		g2.fillRect(0, 0, width, height);
		g2.dispose();
		            
		return gradientImage;
	}

	protected Image getGradientImage()  {
		Dimension size = getSize();
		if (image == null || image.getWidth(null) != size.width || image.getHeight(null) != size.height) 
			{
			image = createGradientImage(size.width, size.height, colors[0], colors[1]);
			}
		return image;
	}

	protected void paintComponent(Graphics g)  {
		final Image gradientImage = getGradientImage();
		g.drawImage(gradientImage, 0, 0, null);
		super.paintComponent(g);
	}

	public void setGradientColor1(Color color)  {
		changeGradientColor(0, color);
	}

	public void setGradientColor2(Color color)  {
		changeGradientColor(1, color);
	}

}
