/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.system.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.plaf.*;

public final class HIconFactory {
	private static final boolean DEBUG = false;

	private static final Hashtable<String, ImageIcon> icons = new Hashtable<String, ImageIcon>();
	private static final ImageIcon noiconfound = HIconFactory.getIcon("noiconfound");
	
	private HIconFactory()  {
	}

	public static ImageIcon getButtonIcon(String name)  {
		return getIcon("button_" + name);
	}

	public static IconUIResource getButtonIcon_UIResource(String name)  {
		final ImageIcon icon = getButtonIcon(name);
		return new IconUIResource(icon);
	}

	public static ImageIcon getFolderIcon(String name)  {
		return getIcon("folder_" + name);
	}

	public static IconUIResource getFolderIcon_UIResource(String name)  {
		final ImageIcon icon = getFolderIcon(name);
		return new IconUIResource(icon);
	}

	public static ImageIcon getIcon(String name)  {
		// try cache
		ImageIcon result = icons.get(name);
		if (result != null) return result;
		
		// load icon
		try
			{
			InputStream resource = Class.forName("de.hillenbrand.swing.HIconFactory").getResourceAsStream("icons/" + name + ".gif");
			if (resource == null) resource = Class.forName("de.hillenbrand.swing.HIconFactory").getResourceAsStream("icons/" + name + ".png");
			if (resource == null) resource = Class.forName("de.hillenbrand.swing.HIconFactory").getResourceAsStream("icons/" + name + ".jpg");
			if (resource == null)
				{
				System.err.println("HIconFactory.getIcon(): icon '"+name+"' not found");
				return noiconfound;
				}
			// read icon
			final BufferedInputStream in = new BufferedInputStream(resource);
			final ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
			byte [] buffer = new byte[1024];
			int n;
			while ((n = in.read(buffer)) > 0) out.write(buffer, 0, n);
			in.close();
			out.flush();
			buffer = out.toByteArray();
			result = new ImageIcon(buffer);
			// put icon in cache
			icons.put(name, result);
			return result;
			}
		catch (Exception x ) { System.err.println("JReqTools: FATAL ERROR " + x); System.exit(10); }
		
		return noiconfound;
	}

	private static String [] getIconNames(final String prefix)  {
		final TreeSet<String> data = new TreeSet<String>();
		
		try
			{
			final Enumeration icons = Class.forName("de.hillenbrand.swing.HIconFactory").getClassLoader().getResources("de/hillenbrand/swing/icons/");
			while (icons.hasMoreElements())
				{
				final URL url = (URL)icons.nextElement();
				final File file = new File(url.getFile());
				if (file.isDirectory())
					{
					final String [] filenames = file.list(new FilenameFilter() {
						public boolean accept(File dir, String fname) {
							return fname.startsWith(prefix);
						}
					});
					for (int i=0; filenames != null && i < filenames.length; ++i) 
						{
						final String s = FileUtils.removeSuffix(filenames[i]);
						data.add(s);
						}
					}
				else 
					{
					String filename = file.getPath();
					filename = filename.substring(5); // "file:" weg
					filename = filename.substring(0, filename.indexOf(".jar!")+4); // "!..." weg
					final ZipFile zip = new ZipFile(filename);
					for (Enumeration enm = zip.entries(); enm.hasMoreElements(); )
						{
						final ZipEntry entry = (ZipEntry) enm.nextElement();
						if (! entry.getName().startsWith("de/hillenbrand/swing/icons/" + prefix)) continue;
						filename = entry.getName();
						filename = filename.substring(27);
						data.add(FileUtils.removeSuffix(filename));
						}
					}
				}
			
			}
		catch (Exception x ) { System.err.println(x); }
		
		return data.toArray(new String[0]);
	}

	public static IconUIResource getIcon_UIResource(String name)  {
		final ImageIcon icon = getIcon(name);
		return new IconUIResource(icon);
	}

	public static ImageIcon getMenuIcon(String name)  {
		return getIcon("16x16_" + name);
	}

	public static IconUIResource getMenuIcon_UIResource(String name)  {
		final ImageIcon icon = getMenuIcon(name);
		return new IconUIResource(icon);
	}

	public static ImageIcon getToolbarIcon(String name)  {
		return getIcon("22x22_" + name);
	}

	public static IconUIResource getToolbarIcon_UIResource(String name)  {
		final ImageIcon icon = getToolbarIcon(name);
		return new IconUIResource(icon);
	}

	public static void main(String [] args)  {
		UIPreferences.init();
		
		final JFrame frame = new JFrame();
		System.out.println(HIconFactory.requestMenuIcon(frame,"fileopen"));
		System.out.println(HIconFactory.requestToolbarIcon(frame,"ok"));
		System.out.println(HIconFactory.requestButtonIcon(frame,"ok"));
		System.out.println(HIconFactory.requestFolderIcon(frame,"blue"));
		
		System.exit(0);
	}

	/**
	* Requests a button icon.
	*/

	public static String requestButtonIcon(JDialog parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestButtonIconMessage"), "button_", defaultIconName != null ? ("button_" + defaultIconName) : null);
		if (result != null) result = result.substring(7);
		return result;
	}

	/**
	* Requests a button icon.
	*/

	public static String requestButtonIcon(JFrame parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestButtonIconMessage"), "button_", defaultIconName != null ? ("button_" + defaultIconName) : null);
		if (result != null) result = result.substring(7);
		return result;
	}

	/**
	* Requests a folder icon.
	*/

	public static String requestFolderIcon(JDialog parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestFolderIconMessage"), "folder_", defaultIconName != null ? ("folder_" + defaultIconName) : null);
		if (result != null) result = result.substring(7);
		return result;
	}

	/**
	* Requests a folder icon.
	*/

	public static String requestFolderIcon(JFrame parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestFolderIconMessage"), "folder_", defaultIconName != null ? ("folder_" + defaultIconName) : null);
		if (result != null) result = result.substring(7);
		return result;
	}

	private static String requestIcon(final JDialog parent, String message, final String prefix, String defaultIconName)  {
		// get icon names
		final String [] iconNames = getIconNames(prefix);
		
		// selected icon
		Icon selected = null;
		
		// create icon array
		final Icon [] icons = new Icon[iconNames.length];
		for (int i=0; i<icons.length; ++i) 
			{
			icons[i] = HIconFactory.getIcon(iconNames[i]);
			if (iconNames[i].equals(defaultIconName)) selected = icons[i];
			}
		
		// request
		final Icon icon = JReqTools.requestIcon(parent, parent.getTitle(), message, icons, selected);
		
		// find icon name
		for (int i=0; i<icons.length; ++i)
			{
			if (icons[i] == icon) return iconNames[i];
			}
		
		return defaultIconName;
	}

	private static String requestIcon(final JFrame parent, String message, final String prefix, String defaultIconName)  {
		// get icon names
		final String [] iconNames = getIconNames(prefix);
		
		// selected icon
		Icon selected = null;
		
		// create icon array
		final Icon [] icons = new Icon[iconNames.length];
		for (int i=0; i<icons.length; ++i) 
			{
			icons[i] = HIconFactory.getIcon(iconNames[i]);
			if (iconNames[i].equals(defaultIconName)) selected = icons[i];
			}
		
		// request
		final Icon icon = JReqTools.requestIcon(parent, parent.getTitle(), message, icons, selected);
		
		// find icon name
		for (int i=0; i<icons.length; ++i)
			{
			if (icons[i] == icon) return iconNames[i];
			}
		
		return defaultIconName;
	}

	/**
	* Request a menu icon.
	*/

	public static String requestMenuIcon(JDialog parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestMenuIconMessage"), "16x16_", defaultIconName != null ? ("16x16_" + defaultIconName) : null);
		if (result != null) result = result.substring(6);
		return result;
	}

	/**
	* Request a menu icon.
	*/

	public static String requestMenuIcon(JFrame parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestMenuIconMessage"), "16x16_", defaultIconName != null ? ("16x16_" + defaultIconName) : null);
		if (result != null) result = result.substring(6);
		return result;
	}

	/**
	* Requests a toolbar button icon.
	*/

	public static String requestToolbarIcon(JDialog parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestToolbarIconMessage"), "22x22_", defaultIconName != null ? ("22x22_" + defaultIconName) : null);
		if (result != null) result = result.substring(6);
		return result;
	}

	/**
	* Requests a toolbar button icon.
	*/

	public static String requestToolbarIcon(JFrame parent, String defaultIconName)  {
		String result = requestIcon(parent, Helper.GRS("HIconFactory.DefaultRequestToolbarIconMessage"), "22x22_", defaultIconName != null ? ("22x22_" + defaultIconName) : null);
		if (result != null) result = result.substring(6);
		return result;
	}

}
