/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import de.hillenbrand.jreqtools.*;
import java.io.File;

public class HItemList<T> extends JPanel
   {
   // Components
   private final JList list = new JList();
   private final JButton add = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_ADD));
   private final JButton sub = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_REMOVE));
   private final JButton up = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_MOVEUP));
   private final JButton down = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_MOVEDOWN));
   private final JScrollPane scrollPane = new JScrollPane(list, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
   private JPopupMenu menu = null;
   // Data
   private final Vector<T> items = new Vector<T>();
   private final Vector<ListSelectionListener> listeners = new Vector<ListSelectionListener>();
   private final Vector<MouseListener> mouseListeners = new Vector<MouseListener>();
   private final Vector<ChangeListener> changeListeners = new Vector<ChangeListener>();
   // Comparator
   private Comparator<T> comparator = new Comparator<T>()
	{
	public int compare(T a, T b)
	   {
	   return a.toString().compareTo(b.toString());
	   }
	};
   private int index = -1;
   private boolean added = false;
   // Property: itemEditor
   protected HItemList.Editor<T> itemEditor = null;

   public interface Editor<T>
	{
	public T[] addItems(JComponent parent);

	public T editItem(JComponent parent, T item);

	public T parseFrom(String s);
	}

   public static class FileItemEditor implements Editor<File>
	{
	final FileRequester f = new FileRequester();

	public FileItemEditor()
	   {
	   f.setOkButtonText("Select");
	   }

	public File[] addItems(JComponent parent)
	   {
	   f.setMultiSelection(true);
	   return f.request(parent) == FileRequester.OK ? f.getFiles() : new File[0];
	   }

	public File editItem(JComponent parent, File item)
	   {
	   f.setMultiSelection(false);
	   if (item != null)
		{
		f.setFile(item);
		}
	   return f.request(parent) == FileRequester.OK ? f.getFile() : item;
	   }

	public File parseFrom(String s)
	   {
	   return new File(s);
	   }
	}

   public static class DirectoryItemEditor extends FileItemEditor
	{
	public DirectoryItemEditor()
	   {
	   f.setDirectoriesOnly(true);
	   }
	}

   public class ListMouseListener implements MouseListener
	{
	public void mouseClicked(java.awt.event.MouseEvent event)
	   {
	   setSelectedIndex(list.locationToIndex(event.getPoint()));
	   event = new MouseEvent(HItemList.this, event.getID(), event.getWhen(), event.getModifiers(), event.getX(), event.getY(), event.getClickCount(), event.isPopupTrigger());
	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseClicked(event);
		}
	   }

	public void mouseEntered(java.awt.event.MouseEvent event)
	   {
	   event = new MouseEvent(HItemList.this, event.getID(), event.getWhen(), event.getModifiers(), event.getX(), event.getY(), event.getClickCount(), event.isPopupTrigger());
	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseEntered(event);
		}
	   }

	public void mouseExited(java.awt.event.MouseEvent event)
	   {
	   event = new MouseEvent(HItemList.this, event.getID(), event.getWhen(), event.getModifiers(), event.getX(), event.getY(), event.getClickCount(), event.isPopupTrigger());
	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseExited(event);
		}
	   }

	public void mousePressed(java.awt.event.MouseEvent event)
	   {
	   event = new MouseEvent(HItemList.this, event.getID(), event.getWhen(), event.getModifiers(), event.getX(), event.getY(), event.getClickCount(), event.isPopupTrigger());
	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mousePressed(event);
		}
	   }

	public void mouseReleased(java.awt.event.MouseEvent event)
	   {
	   event = new MouseEvent(HItemList.this, event.getID(), event.getWhen(), event.getModifiers(), event.getX(), event.getY(), event.getClickCount(), event.isPopupTrigger());
	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseReleased(event);
		}
	   }
	}

   public HItemList()
	{
	this(null);
	}

   public HItemList(T[] data)
	{
	super(new GridBagLayout());

	// Properties
	up.setMargin(new Insets(1, 1, 1, 1));
	down.setMargin(new Insets(1, 1, 1, 1));
	add.setMargin(new Insets(1, 1, 1, 1));
	sub.setMargin(new Insets(1, 1, 1, 1));

	// Tooltips
	add.setToolTipText(Helper.GRS("HEditableList.ToolTip.Add"));
	sub.setToolTipText(Helper.GRS("HEditableList.ToolTip.Remove"));
	up.setToolTipText(Helper.GRS("HEditableList.ToolTip.Up"));
	down.setToolTipText(Helper.GRS("HEditableList.ToolTip.Down"));

	// Listeners
	add.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		action_addItem();
		}
	   });
	sub.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		removeSelectedItem();
		}
	   });
	up.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		moveSelectedItemUp();
		}
	   });
	down.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		moveSelectedItemDown();
		}
	   });
	list.addMouseListener(new HItemList.ListMouseListener());

	// Create Panel
	final GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(0, 0, 0, 0);

	// JList
	c.fill = GridBagConstraints.BOTH;
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 5;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 1;
	this.add(new JScrollPane(list, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);

	// HTextField
	c.fill = GridBagConstraints.BOTH;
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	this.add(new JPanel(), c);

	// JButtons
	c.fill = GridBagConstraints.NONE;
	c.gridx = 1;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(add, c);
	c.gridx = 2;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(sub, c);
	c.gridx = 3;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(up, c);
	c.gridx = 4;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(down, c);

	// Menus
	menu = SwingUtils.createPopupMenu(list, "Items");
	JMenuItem item = menu.add(new JMenuItem(Helper.GRS("HEditableList.Sort"), UIManager.getIcon(UIPreferences.MENU_ICON_SORT)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		sortItems();
		}
	   });
	item = menu.add(new JMenuItem(Helper.GRS("HEditableList.Clear"), UIManager.getIcon(UIPreferences.MENU_ICON_CLEAR)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		removeAllItems();
		}
	   });

	// Initialisieren mit Daten
	if (data != null)
	   {
	   setItems(data);
	   }
	}

   private void action_addItem()
	{
	if (itemEditor != null)
	   {
	   final T[] result = itemEditor.addItems(this);
	   for (T i : result)
		{
		addItem(i);
		
		}
	   }
	else
	   {
	   JReqTools.showError(SwingUtils.getJFrame(this), "No editor available.");
	   }
	}

   public void addChangeListener(ChangeListener l)
	{
	changeListeners.addElement(l);
	}

   public synchronized void addItem(T item)
	{
	items.addElement(item);
	updateJList();
	sendChangeEvent();
	}

   public synchronized void addItems(Collection<T> data)
	{
	for (T t : data)
	   {
	   items.addElement(t);
	   }
	index = -1;

	updateJList();
	sendChangeEvent();
	}

   public synchronized void addItems(T[] data)
	{
	for (T t : data)
	   {
	   items.addElement(t);
	   }
	index = -1;

	updateJList();
	sendChangeEvent();
	}

   public void addListSelectionListener(ListSelectionListener l)
	{
	listeners.addElement(l);
	}

   public synchronized JMenuItem addMenuItem(JMenuItem item)
	{
	if (!added)
	   {
	   menu.addSeparator();
	   added = true;
	   }

	menu.add(item);
	return item;
	}

   public synchronized JMenuItem addMenuItem(final T item, Icon icon)
	{
	final JMenuItem menuItem = new JMenuItem(item.toString(), icon);
	menuItem.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		addItem(item);
		}
	   });
	addMenuItem(menuItem);
	return menuItem;
	}

   public void addMouseListener(MouseListener l)
	{
	mouseListeners.addElement(l);
	}

   public synchronized void changeSelectedItem(T item)
	{
	final int ixd = list.getSelectedIndex();
	if (ixd == -1)
	   {
	   return;
	   }

	items.setElementAt(item, ixd);
	updateJList();

	sendChangeEvent();
	}

   public synchronized boolean contains(T item)
	{
	return items.contains(item);
	}

   /**
    * Returns the value of the cellRenderer property.
    * @return
    */
   public ListCellRenderer getCellRenderer()
	{
	return list.getCellRenderer();
	}

   /**
    * Returns the value of the itemEditor property.
    * @return
    */
   public HItemList.Editor<T> getItemEditor()
	{
	return itemEditor;
	}

   public synchronized T[] getItems(T[] a)
	{
	@SuppressWarnings("unchecked")
	final T[] result = (T[]) java.lang.reflect.Array.newInstance(a.getClass().getComponentType(), items.size());
	for (int i = 0; i < result.length; ++i)
	   {
	   result[i] = items.elementAt(i);
	   }
	return result;
	}

   /**
    * Returns the JPoupMenu associated with this component. Be careful using it!
    * @return
    */
   public JPopupMenu getPopupMenu()
	{
	return menu;
	}

   public synchronized int getSelectedIndex()
	{
	return list.getSelectedIndex();
	}

   public synchronized T getSelectedItem()
	{
	int idx = getSelectedIndex();
	if (idx == -1)
	   {
	   return null;
	   }
	return items.elementAt(idx);
	}

   public synchronized int length()
	{
	return items.size();
	}

   public synchronized void moveSelectedItemDown()
	{
	int idx = list.getSelectedIndex();
	if (idx == -1 || idx == items.size() - 1)
	   {
	   SwingUtils.beep();
	   return;
	   }

	final T s = items.elementAt(idx);
	items.removeElementAt(idx);
	items.insertElementAt(s, ++idx);

	this.index = idx;

	updateJList();
	sendChangeEvent();
	}

   public synchronized void moveSelectedItemUp()
	{
	int idx = list.getSelectedIndex();
	if (idx == -1 || idx == 0)
	   {
	   SwingUtils.beep();
	   return;
	   }

	final T s = items.elementAt(idx);
	items.removeElementAt(idx);
	items.insertElementAt(s, --idx);

	this.index = idx;

	updateJList();
	sendChangeEvent();
	}

   public synchronized void removeAllItems()
	{
	items.setSize(0);
	index = -1;

	updateJList();
	sendChangeEvent();
	}

   public synchronized void removeAllMenuItems()
	{
	while (menu.getComponentCount() > 2)
	   {
	   menu.remove(2);
	   }
	}

   public synchronized void removeSelectedItem()
	{
	int idx = list.getSelectedIndex();
	if (idx == -1)
	   {
	   SwingUtils.beep();
	   return;
	   }

	final T s = items.elementAt(idx);
	items.removeElementAt(idx);
	this.index = Math.min(idx, items.size() - 1);

	updateJList();
	sendChangeEvent();
	}

   private void sendChangeEvent()
	{
	for (Enumeration e = changeListeners.elements(); e.hasMoreElements();)
	   {
	   ((ChangeListener) e.nextElement()).stateChanged(new ChangeEvent(this));
	   }
	}

   /**
    * Assigns the value of the cellRenderer property.
    * @param arg
    */
   public void setCellRenderer(ListCellRenderer arg)
	{
	list.setCellRenderer(arg);
	}

   public void setChangeEvent()
	{
	}

   public void setComparator(Comparator<T> comp)
	{
	comparator = comp;
	}

   /**
    * Assigns the value of the itemEditor property.
    * @param arg
    */
   public void setItemEditor(HItemList.Editor<T> arg)
	{
	itemEditor = arg;
	}

   public synchronized void setItems(Collection<T> data)
	{
	if (data != null)
	   {
	   items.setSize(0);
	   for (T t : data)
		{
		items.addElement(t);
		}
	   index = data.size() == 0 ? -1 : 0;
	   updateJList();
	   sendChangeEvent();
	   }
	else
	   {
	   removeAllItems();
	   }
	}

   public synchronized void setItems(T[] data)
	{
	if (data != null)
	   {
	   items.setSize(0);
	   for (T t : data)
		{
		items.addElement(t);
		}
	   index = data.length == 0 ? -1 : 0;
	   updateJList();
	   sendChangeEvent();
	   }
	else
	   {
	   removeAllItems();
	   }
	}

   public void setLabel(String text)
	{
	add(new JLabel(text), 0);
	}

   public synchronized void setSelectedIndex(int value)
	{
	index = value;
	updateJList();
	}

   public synchronized void setSelectedItem(String text)
	{
	index = items.indexOf(text);
	updateJList();
	}

   public void setToolTipText(String text)
	{
	list.setToolTipText(text);
	}

   @SuppressWarnings(
	{
	"unchecked"
	})
   public synchronized void sortItems()
	{
	if (items.size() > 0 && items.elementAt(0) instanceof Comparable)
	   {
	   final Object[] toSort = this.items.toArray();
	   Arrays.sort(toSort);
	   setItems((T[]) toSort);
	   }
	else
	   {
	   final Object[] toSort = this.items.toArray();
	   Arrays.sort((T[]) toSort, comparator);
	   setItems((T[]) toSort);
	   }
	}

   private void updateJList()
	{
	list.setListData(items);
	list.repaint();

	if (index != -1)
	   {
	   list.setSelectedIndex(index);
	   }

	scrollPane.validate();
	}

   public void valueChanged(javax.swing.event.ListSelectionEvent ev)
	{
	ev = new ListSelectionEvent(HItemList.this, ev.getFirstIndex(), ev.getLastIndex(), ev.getValueIsAdjusting());
	for (Enumeration e = listeners.elements(); e.hasMoreElements();)
	   {
	   ((ListSelectionListener) e.nextElement()).valueChanged(ev);
	   }
	}

   public String[] getItemsAsStringArray()
	{
	final ArrayList<String> result = new ArrayList<String>();
	for (T item : items)
	   {
	   result.add(item.toString());
	   }
	return result.toArray(new String[0]);
	}

   public synchronized void setItemsAsStringArray(String[] items)
	{
	if (itemEditor == null)
	   {
	   throw new IllegalArgumentException("No item editor for converting from String to T available.");
	   }

	removeAllItems();
	for (String item : items)
	   {
	   addItem(itemEditor.parseFrom(item));
	   }
	}
   }
