package de.hillenbrand.swing;

import de.hillenbrand.jreqtools.JReqTools;
import de.hillenbrand.system.UIPreferences;
import de.hillenbrand.util.FileUtils;
import de.hillenbrand.util.StringUtils;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import javax.swing.UIManager;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.Element;

public class HTextEditor extends HPanel
   {
   private final HEditorPane editor = new HEditorPane();
   private final HButton reload = new HButton("Reload", UIManager.getIcon(UIPreferences.BUTTON_ICON_RELOAD), "Reload from file.");
   private final HButton save = new HButton("Save", UIManager.getIcon(UIPreferences.BUTTON_ICON_SAVE), "Save file.");
   private final HButton externalEditor = new HButton("Editor", UIManager.getIcon(UIPreferences.BUTTON_ICON_SAVE), "Edit using external editor.");
   private final HLabel label = new HLabel("n/a", HLabel.LEFT);
   private final HButtonPanel buttons;
   // volatile data
   private boolean modified = false;
   private boolean autoRequestModification = false;
   // data
   private File file;

   public HTextEditor()
	{
	this(null, new String[0]);
	}

   public HTextEditor(File file)
	{
	this(file, new String[0]);
	}

   public HTextEditor(File file, String[] menuEntries)
	{
	super(new GridBagLayout());

	this.file = file;
	editor.setTabSize(3);
	label.setFont(UIPreferences.getAcceleratorFont());
	label.setForeground(Color.GRAY);

	// add menu entries
	for (final String s : menuEntries)
	   {
	   final HMenuItem item = editor.getHPopupMenu().add(new HMenuItem(s));
	   item.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   editor.insertText(s + "\n");
		   }
		});
	   }

	// listeners
	externalEditor.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		action_edit();
		}
	   });
	reload.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		action_reload();
		}
	   });
	save.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		action_save();
		}
	   });
	editor.getDocument().addDocumentListener(new DocumentListener()
	   {
	   public void insertUpdate(DocumentEvent e)
		{
		modified = true;
		}

	   public void removeUpdate(DocumentEvent e)
		{
		modified = true;
		}

	   public void changedUpdate(DocumentEvent e)
		{
		modified = true;
		}
	   });

	// add components
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();
	c.set(0, 1, 1, 1, 1, 1, HGridBagConstraints.BOTH, HGridBagConstraints.ZERO_INSETS);
	this.add(new HScrollPane(editor), c);
	c.set(0, 2, 1, 1, 1, 0, HGridBagConstraints.HORIZONTAL, HGridBagConstraints.ZERO_INSETS);
	this.add(buttons = new HButtonPanel(label, externalEditor, reload, save), c);

	// initial update
	action_reload();
	}

   public File getFile()
	{
	return file;
	}

   public String getText()
	{
	return editor.getText();
	}

   public void setFile(File file)
	{
	if (modified && autoRequestModification)
	   {
	   final boolean ok = JReqTools.showQuestion(this, this.file + "\nhas been modified. Do you want to save it?");
	   if (ok)
		{
		action_save();
		}
	   }

	this.file = file;
	action_reload();
	}

   public void addButton(String label, Icon icon, String tooltip, ActionListener l)
	{
	final HButton b = new HButton(label, icon, tooltip);
	b.addActionListener(l);
	buttons.addJComponent(1, b);
	}

   protected void action_reload()
	{
	reload.setEnabled(file != null);
	save.setEnabled(file != null);
	editor.setEnabled(file != null);
	modified = false;

	if (file != null)
	   {
	   try
		{
		final String data = StringUtils.readStringFromFile(file);
		editor.setText(data);
		editor.setCaretPosition(0);
		editor.clearUndoRedo();
		setStatus();
		}
	   catch (FileNotFoundException x)
		{
		editor.setText("");
		editor.setCaretPosition(0);
		editor.clearUndoRedo();
		label.setText("File not found: " + file);
		}
	   catch (IOException x)
		{
		JReqTools.showException(this, x);
		}
	   }
	else
	   {
	   editor.setText("");
	   setStatus();
	   }

	modified = false;
	}

   protected void action_save()
	{
	try
	   {
	   FileUtils.writeToFile(file, editor.getText());
	   modified = false;
	   setStatus();
	   }
	catch (IOException x)
	   {
	   JReqTools.showException(this, x);
	   }
	}

   protected void action_edit()
	{
	try
	   {
	   final Process process = HDesktop.edit(this, file);
	   process.waitFor();
	   action_reload();
	   }
	catch (InterruptedException ex)
	   {
	   Logger.getLogger(HTextEditor.class.getName()).log(Level.SEVERE, null, ex);
	   }
	}

   protected void setStatus()
	{
	if (file != null)
	   {
	   final long lastModified = file.lastModified();
	   final Date date = new Date(lastModified);
	   label.setText(file + " [" + date + ", " + file.length() + " bytes]");
	   }
	else
	   {
	   label.setText("No file loaded.");
	   }
	}

   public boolean isModified()
	{
	return modified;
	}

   public boolean isAutoRequestModification()
	{
	return autoRequestModification;
	}

   public void setAutoRequestModification(boolean autoRequestModification)
	{
	this.autoRequestModification = autoRequestModification;
	}

   public void setCaretPosition(int pos)
	{
	editor.setCaretPosition(pos);
	}

   public void select(int start, int end)
	{
	editor.select(start, end);
	}

   public void selectLine(int line)
	{
	try
	   {
	   final Document d = editor.getDocument();
	   final Element e = d.getDefaultRootElement().getElement(line);
	   requestFocusInWindow(true);
	   select(e.getStartOffset(), e.getEndOffset());
	   }
	catch (Exception x)
	   {
	   Logger.getLogger(HTextEditor.class.getName()).log(Level.SEVERE, null, x);
	   }
	}

   public void insertText(String text)
	{
	editor.insertText(text);
	}
   }
