/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.io.*;
import de.hillenbrand.jreqtools.JReqTools;
import de.hillenbrand.system.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.reflect.*;
import java.security.*;
import java.util.*;
import java.util.logging.Level;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

public final class SwingUtils
   {
   private static final boolean DEBUG = false;
   public static final Cursor NORMAL_CURSOR = new Cursor(Cursor.DEFAULT_CURSOR);
   public static final Cursor WAIT_CURSOR = new Cursor(Cursor.WAIT_CURSOR);

   // this dummy class is a workaround to get the currently focused text component
   private final static class DummyTextAction extends TextAction
	{
	public DummyTextAction()
	   {
	   super("Dummy");
	   }

	public void actionPerformed(ActionEvent e)
	   {
	   }

	public JTextComponent getIt()
	   {
	   return getFocusedComponent();
	   }
	};
   private static final DummyTextAction dummyTextAction = new DummyTextAction();

   SwingUtils()
	{
	}

   private static String SPACE(int len)
	{
	String s = "";
	for (int i = 0; i < len; ++i)
	   {
	   s += " ";
	   }
	return s;
	}

   @SuppressWarnings("unchecked")
   private static void _call(Object klass, String method)
	{
	try
	   {
	   final Object obj = AccessController.doPrivileged(new HInternalDialog.ModalPrivilegedAction(klass.getClass(), method));
	   if (obj != null)
		{
		((Method) obj).invoke(klass);
		}
	   else
		{
		System.err.println("Method '" + method + "' not found in class '" + klass + "'.");
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }
	}

   public static void addHook_FrameClosed(final Container c, final Object klass, final String method)
	{
	if (c instanceof JComponent && SwingUtils.getJInternalFrame((JComponent) c) != null)
	   {
	   final JInternalFrame f = SwingUtils.getJInternalFrame((JComponent) c);
	   f.addInternalFrameListener(new InternalFrameListener()
		{
		public void internalFrameClosed(InternalFrameEvent e)
		   {
		   _call(klass, method);
		   }

		public void internalFrameActivated(InternalFrameEvent e)
		   {
		   }

		public void internalFrameClosing(InternalFrameEvent e)
		   {
		   }

		public void internalFrameDeactivated(InternalFrameEvent e)
		   {
		   }

		public void internalFrameDeiconified(InternalFrameEvent e)
		   {
		   }

		public void internalFrameIconified(InternalFrameEvent e)
		   {
		   }

		public void internalFrameOpened(InternalFrameEvent e)
		   {
		   }
		});
	   }
	else if (c instanceof JComponent && SwingUtils.getJFrame((JComponent) c) != null)
	   {
	   final JFrame f = SwingUtils.getJFrame((JComponent) c);
	   f.addWindowListener(new WindowAdapter()
		{
		public void windowClosed(WindowEvent e)
		   {
		   _call(klass, method);
		   }
		});
	   }
	else
	   {
	   throw new IllegalArgumentException("Component type '" + c.getClass().getName() + "' not supported by SwingUtils.addHook_ComponentShown");
	   }
	}

   public static void addJPopupMenuToJComponent(JPopupMenu p, JComponent c)
	{
	c.addMouseListener(SwingUtils.createMouseListenerForPopupMenu(c, p));
	}

   public static void addMenuItemsFromFile(String file, String prefix, String suffix, JPopupMenu menu, ActionListener listener)
	{
	addMenuItemsFromFile(file, prefix, suffix, menu, listener);
	}

   public static void addMenuItemsFromFile(String file, String prefix, String suffix, String iconname, JPopupMenu menu, ActionListener listener)
	{
	if (prefix == null)
	   {
	   prefix = "";
	   }
	if (suffix == null)
	   {
	   suffix = "";
	   }

	try
	   {
	   final FileAccess f = new FileAccess(file);
	   String line;
	   while ((line = f.readLine()) != null)
		{
		final JMenuItem item = menu.add(new JMenuItem(prefix + line + suffix));
		item.setActionCommand(line);
		item.addActionListener(listener);
		if (iconname != null)
		   {
		   item.setIcon(HIconFactory.getMenuIcon(iconname));
		   }
		}
	   }
	catch (IOException x)
	   {
	   System.err.println(x);
	   }
	}

   public static void addMenuItemsFromStringArray(String[] data, String prefix, String suffix, JPopupMenu menu, ActionListener listener)
	{
	addMenuItemsFromStringArray(data, prefix, suffix, null, menu, listener);
	}

   public static void addMenuItemsFromStringArray(String[] data, String prefix, String suffix, String iconname, JPopupMenu menu, ActionListener listener)
	{
	if (prefix == null)
	   {
	   prefix = "";
	   }
	if (suffix == null)
	   {
	   suffix = "";
	   }

	for (int i = 0; i < data.length; ++i)
	   {
	   final JMenuItem item = menu.add(new JMenuItem(prefix + data[i] + suffix));
	   item.setActionCommand(data[i]);
	   item.addActionListener(listener);
	   if (iconname != null)
		{
		item.setIcon(HIconFactory.getMenuIcon(iconname));
		}
	   }
	}

   /**
    * Plays the default error sound if set, or plays a beep.
    */
   public static void beep()
	{
	final String filename = UIPreferences.getErrorSound();
	final File file = new File(filename);
	if (file.exists() && file.isFile())
	   {
	   AudioUtils.playSound(filename);
	   }
	else
	   {
	   Toolkit.getDefaultToolkit().beep();
	   }
	}

   public static void centerJDialogOnJFrame(JFrame parent, JDialog toCenter)
	{
	final Dimension pdim = parent.getSize();
	final Dimension dim = toCenter.getSize();
	final Point location = parent.getLocationOnScreen();

	location.x = location.x + (pdim.width - dim.width) / 2;
	if (location.x < 0)
	   {
	   location.x = 0;
	   }

	location.y = location.y + (pdim.height - dim.height) / 2;
	if (location.y < 0)
	   {
	   location.y = 0;
	   }

	toCenter.setLocation(location);
	}

   public static void centerJFrameOnJFrame(JFrame parent, JFrame toCenter)
	{
	final Dimension pdim = parent.getSize();
	final Dimension dim = toCenter.getSize();
	final Point location = parent.getLocationOnScreen();

	location.x = location.x + (pdim.width - dim.width) / 2;
	if (location.x < 0)
	   {
	   location.x = 0;
	   }

	location.y = location.y + (pdim.height - dim.height) / 2;
	if (location.y < 0)
	   {
	   location.y = 0;
	   }

	toCenter.setLocation(location);
	}

   public static void centerJFrameOnScreen(JFrame toCenter)
	{
	centerOnPrimaryDisplay(toCenter);
	}

   /**
    * Centers component toCenter on a parent container, e.g. a JDialog on a JFrame.
    */
   public static Point centerOn(Container parent, Component toCenter)
	{
	final Rectangle parentBounds = parent.getBounds();
	final Dimension size = toCenter.getSize();

	int x = parentBounds.x + (parentBounds.width - size.width) / 2;
	if (x < 0)
	   {
	   x = 0;
	   }

	int y = parentBounds.y + (parentBounds.height - size.height) / 2;
	if (y < 0)
	   {
	   y = 0;
	   }

	toCenter.setLocation(x, y);
	return toCenter.getLocation();
	}

   /**
    * Centers component toCenter on a parent container, e.g. a JDialog on a JFrame.
    */
   public static Point centerOn(Container parent, HDialog toCenter)
	{
	final Rectangle parentBounds = parent.getBounds();
	final Dimension size = toCenter.getSize();

	int x = parentBounds.x + (parentBounds.width - size.width) / 2;
	if (x < 0)
	   {
	   x = 0;
	   }

	int y = parentBounds.y + (parentBounds.height - size.height) / 2;
	if (y < 0)
	   {
	   y = 0;
	   }

	toCenter.setLocation(x, y);
	return toCenter.getLocation();
	}

   public static void centerOnPrimaryDisplay(java.awt.Window w)
	{
	final GraphicsDevice primary = UIPreferences.getPrimaryGraphicsDevice();
	final Rectangle bounds = primary.getDefaultConfiguration().getBounds();
	final Dimension size = w.getSize();
	w.setLocation(bounds.x + (bounds.width - size.width) / 2, bounds.y + (bounds.height - size.height) / 2);
	}

   public static Font createFontFromString(String s)
	{
	int index = s.indexOf("/");
	return new Font(s.substring(0, index), Font.PLAIN, Integer.parseInt(s.substring(index + 1)));
	}

   public static MouseListener createMouseListenerForPopupMenu(final JComponent c, final JPopupMenu menu)
	{
	if (SwingUtils.isWindowsSystem())
	   {
	   return new MouseAdapter()
		{
		public void mouseReleased(MouseEvent e)
		   {
		   if (e.isPopupTrigger())
			{
			menu.show(c, e.getX() - 5, e.getY() - 5);
			e.consume();
			}
		   }
		};
	   }
	else
	   {
	   return new MouseAdapter()
		{
		public void mousePressed(MouseEvent e)
		   {
		   if (e.isPopupTrigger())
			{
			menu.show(c, e.getX(), e.getY());
			e.consume();
			}
		   }
		};
	   }
	}

   public static JPopupMenu createPopupMenu(JComponent parent, String title)
	{
	final JPopupMenu result = new JPopupMenu(title);
	parent.addMouseListener(SwingUtils.createMouseListenerForPopupMenu(parent, result));

	return result;
	}

   /**
    * Returns the JTextComponent that had or has the focus.
    */
   public static JTextComponent getFocusedJTextComponent()
	{
	return dummyTextAction.getIt();
	}

   /**
    * Gets the frame this components is in.
    */
   public static JDesktopPane getJDesktopPane(JComponent comp)
	{
	if (comp == null)
	   {
	   return null;
	   }
	Container c = comp.getParent();

	while (c != null)
	   {
	   if (c instanceof JDesktopPane)
		{
		return (JDesktopPane) c;
		}
	   c = c.getParent();
	   }

	return null;
	}

   /**
    * Gets the frame this components is in.
    */
   public static JFrame getJFrame(JComponent comp)
	{
	if (comp == null)
	   {
	   return null;
	   }
	Container c = comp.getParent();

	while (c != null)
	   {
	   if (c instanceof JFrame)
		{
		return (JFrame) c;
		}
	   c = c.getParent();
	   }

	return null;
	}

   public static JInternalFrame getJInternalFrame(JComponent comp)
	{
	if (comp == null)
	   {
	   return null;
	   }
	Container c = comp.getParent();

	while (c != null)
	   {
	   if (c instanceof JInternalFrame)
		{
		return (JInternalFrame) c;
		}
	   c = c.getParent();
	   }

	return null;
	}

   public static boolean isWindowsSystem()
	{
	String osName = System.getProperty("os.name");
	return (osName != null) && (osName.indexOf("Windows") != -1);
	}

   public static void printComponents(java.awt.Container c, int offset)
	{
	for (int i = 0; i < c.getComponentCount(); i = i + 1)
	   {
	   final Component comp = c.getComponent(i);
	   System.out.println(SPACE(offset) + "- " + comp.getClass().getName());
	   if (comp instanceof Container)
		{
		printComponents((Container) comp, offset + 1);
		}
	   }

	}

   public static void setBusy(Container parent, boolean bool)
	{
	final Vector<Container> containers = new Vector<Container>();
	containers.addElement(parent);

	while (containers.size() > 0)
	   {
	   final Container c = containers.elementAt(0);
	   containers.removeElementAt(0);
	   int len = c.getComponentCount();
	   for (int i = 0; i < len; i++)
		{
		final Component comp = c.getComponent(i);
		if (comp instanceof Container)
		   {
		   containers.addElement((Container) comp);
		   }
		comp.setCursor(bool ? WAIT_CURSOR : NORMAL_CURSOR);
		}
	   }
	}

   public static void setEnabledRecursively(Component comp, boolean enabled)
	{
	if (DEBUG)
	   {
	   System.out.println("SwingUtils.setEnabledRecursively(): " + comp.getClass().getName());
	   }

	if (comp instanceof JSplitPane)
	   {
	   comp.setEnabled(enabled);
	   setEnabledRecursively(((JSplitPane) comp).getLeftComponent(), enabled);
	   setEnabledRecursively(((JSplitPane) comp).getRightComponent(), enabled);
	   }
	else if (comp instanceof JTabbedPane)
	   {
	   comp.setEnabled(enabled);
	   final int len = ((JTabbedPane) comp).getTabCount();
	   for (int i = 0; i < len; i++)
		{
		final Component c = ((JTabbedPane) comp).getComponentAt(i);
		setEnabledRecursively(c, enabled);
		}
	   }
	else if (comp instanceof JScrollPane)
	   {
	   comp.setEnabled(enabled);
	   setEnabledRecursively(((JScrollPane) comp).getViewport().getView(), enabled);
	   }
	else if (comp instanceof JPanel)
	   {
	   final int len = ((JPanel) comp).getComponentCount();
	   for (int i = 0; i < len; i++)
		{
		final Component c = ((JPanel) comp).getComponent(i);
		setEnabledRecursively(c, enabled);
		}
	   }
	else if (comp instanceof JLabel)
	   {
	   }
	else if (comp instanceof JComponent)
	   {
	   comp.setEnabled(enabled);
	   }
	else
	   {
	   System.err.println("SwingUtils.setEnabledRecursively(): unknown type '" + comp.getClass().getName() + "'");
	   }
	}

   private static JComponent getParentComponent(Object o)
	{
	if (o instanceof JComponent)
	   {
	   return (JComponent) o;
	   }
	else
	   {
	   return null;
	   }
	}

   public static Action createAction(final Object parentClass, final String methodName)
	{
	final ActionListener listener = createActionListener(parentClass, methodName);
	final Action result = new AbstractAction(methodName)
	   {
	   public void actionPerformed(ActionEvent e)
		{
		listener.actionPerformed(e);
		}
	   };
	return result;
	}

   public static ActionListener createActionListener(final Object parentClass, final String methodName)
	{
	try
	   {
	   parentClass.getClass().getDeclaredMethod(methodName);
	   }
	catch (NoSuchMethodException x)
	   {
	   java.util.logging.Logger.getLogger(SwingUtils.class.getName()).log(Level.WARNING, "No such method: {0}.{1}()", new Object[]{parentClass.getClass().getName(), methodName});
	   }

	final ActionListener result = new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		try
		   {
		   final Object method = AccessController.doPrivileged(new HPrivilegedAction(parentClass.getClass(), methodName));
		   if (method != null)
			{
			((Method) method).invoke(parentClass);
			}
		   else
			{
			throw new NoSuchMethodException(parentClass.getClass().getName() + "." + methodName + "()");
			}
		   }
		catch (Exception x)
		   {
		   JReqTools.showException(getParentComponent(parentClass), x);
		   }
		}
	   };
	return result;
	}

   public static MouseListener createMouseClickedListener(final int clickCount, final Object parentClass, final String methodName)
	{
	final MouseListener result = new MouseAdapter()
	   {
	   public void mouseClicked(MouseEvent e)
		{
		if (e.getClickCount() == clickCount)
		   {
		   try
			{
			final Object method = AccessController.doPrivileged(new HPrivilegedAction(parentClass.getClass(), methodName));
			if (method != null)
			   {
			   ((Method) method).invoke(parentClass);
			   }
			else
			   {
			   throw new NoSuchMethodException(parentClass.getClass().getName() + "." + methodName + "()");
			   }
			}
		   catch (Exception x)
			{
			JReqTools.showException(getParentComponent(parentClass), x);
			}
		   }
		}
	   };
	return result;
	}
   }
