/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing.renderer;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A generic list cell renderer with an icon, a title and several tag/value pairs.
 * @param <T>
 */
public abstract class MultiLineListCellRenderer<T extends Object> extends HPanel implements ListCellRenderer
   {
   private final Border noFocusBorder;
   private final Color color = getColor();
   private final HLabel icon;
   private final HLabel title = new HLabel();
   private final HLabel[] names;
   private final HLabel[] values;

   /**
    * Constructs a new list cell renderer.
    * @param	icon Icon to display on the left side
    * @param	tagNames name of each additional line
    */
   public MultiLineListCellRenderer(Icon icon, String... tagNames)
	{
	super(new GridBagLayout());

	// init
	this.icon = new HLabel(icon);

	// create no focus border
	final Border border = UIManager.getBorder("List.focusCellHighlightBorder");
	final Insets borderInsets = border.getBorderInsets(this);
	noFocusBorder = new EmptyBorder(borderInsets.top, borderInsets.right, borderInsets.bottom, borderInsets.left);

	// init
	setOpaque(true);
	setBorder(noFocusBorder);

	// create labels
	names = new HLabel[tagNames.length];
	values = new HLabel[tagNames.length];

	// init labels
	for (int i = 0; i < tagNames.length; ++i)
	   {
	   names[i] = new HLabel(tagNames[i]);
	   names[i].setForeground(color);
	   names[i].setFont(UIPreferences.getAcceleratorFont());
	   values[i] = new HLabel("n/a");
	   values[i].setForeground(color);
	   values[i].setFont(UIPreferences.getAcceleratorFont());
	   }

	// create constraints
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();
	c.set(0, 0, 1, 2 + tagNames.length, 0, 0, GridBagConstraints.NONE, new Insets(0, 2, 0, 2));
	this.add(this.icon, c);
	c.set(1, 0, 2, 1, 1, 0, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 0, 2));
	this.add(title, c);

	for (int i = 0; i < tagNames.length; ++i)
	   {
	   c.set(1, 1 + i, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 0, 2));
	   this.add(names[i], c);
	   c.set(2, 1 + i, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL, new Insets(0, 2, 0, 2));
	   this.add(values[i], c);
	   }

	c.set(1, 1 + tagNames.length, 2, 1, 1, 1, GridBagConstraints.BOTH, new Insets(0, 2, 0, 2));
	this.add(new HLabel(), c);
	}

   /**
    * The color of the text.
    * @return
    */
   protected Color getColor()
	{
	return new Color(100, 100, 100);
	}

   /**
    * Update/Format the component according to the value's actual type.
    * @param list
    * @param value
    * @param index
    * @param isSelected
    * @param cellHasFocus
    */
   protected abstract void formatListCellRendererComponent(JList list, T value, int index, boolean isSelected, boolean cellHasFocus);

   /**
    * Creates the JComponent to be displayed inside the JList.
    *
    * @param	list the list
    * @param	value the current value
    * @param	index index of the value in the list
    * @param	isSelected is the value selected
    * @param	cellHasFocus does the cell currenty have the focus
    * @return
    */
   @SuppressWarnings("unchecked")
   public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
	{
	// title
	title.setText(getTitle((T) value));

	// values
	for (int i = 0; i < values.length; ++i)
	   {
	   values[i].setText(getValue(i, (T) value));
	   }

	setComponentOrientation(list.getComponentOrientation());

	if (isSelected)
	   {
	   setBackground(list.getSelectionBackground());
	   setForeground(list.getSelectionForeground());
	   }
	else
	   {
	   setBackground(list.getBackground());
	   setForeground(list.getForeground());
	   }

	setEnabled(list.isEnabled());
	setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);

	formatListCellRendererComponent(list, (T) value, index, isSelected, cellHasFocus);

	return this;
	}

   /**
    * The title (i.e. the topmost line) of the entry.
    *
    * @param	value the value currently displayed
    * @return
    */
   protected abstract String getTitle(T value);

   /**
    * The tag's value on position pos.
    *
    * @param	pos tag position
    * @param	value the currently selected value
    * @return
    */
   protected abstract String getValue(int pos, T value);

   public void setIcon(Icon i)
	{
	icon.setIcon(i);
	}

   /**
    * Sets the tag name.
    *
    * @param	pos position/index of the tag
    * @param	text the new text to be displayed as tag name
    */
   protected void setTagName(int pos, String text)
	{
	names[pos].setText(text);
	}
   }
