/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing.text;

import java.awt.*;
import javax.swing.text.*;

/**
 * A collection of styles used to render java text.
 * This class also acts as a factory for the views used
 * to represent the java documents.  Since the rendering
 * styles are based upon view preferences, the views need
 * a way to gain access to the style settings which is
 * facilitated by implementing the factory in the style
 * storage.  Both functionalities can be widely shared across
 * java document views.
 *
 * @author   Timothy Prinzing
 * @version  1.2 05/04/98
 */
public class JavaContext extends StyleContext implements ViewFactory
   {
   /**
    * The styles representing the actual token types.
    */
   Style[] tokenStyles;
   /**
    * Cache of foreground colors to represent the
    * various tokens.
    */
   transient Color[] tokenColors;
   /**
    * Cache of fonts to represent the various tokens.
    */
   transient Font[] tokenFonts;

   /**
    * View that uses the lexical information to determine the
    * style characteristics of the text that it renders.  This
    * simply colorizes the various tokens and assumes a constant
    * font family and size.
    */
   class JavaView extends WrappedPlainView
	{
	JavaDocument.Scanner lexer;
	boolean lexerValid;

	/**
	 * Construct a simple colorized view of java
	 * text.
	 */
	JavaView(Element elem)
	   {
	   super(elem);

	   Document doc = getDocument();
	   if (doc instanceof JavaDocument)
		{
		lexer = ((JavaDocument) doc).createScanner();
		lexerValid = false;
		}
	   }

	/**
	 * Renders the given range in the model as normal unselected
	 * text.  This is implemented to paint colors based upon the
	 * token-to-color translations.  To reduce the number of calls
	 * to the Graphics object, text is batched up until a color
	 * change is detected or the entire requested range has been
	 * reached.
	 *
	 * @param g the graphics context
	 * @param x the starting X coordinate
	 * @param y the starting Y coordinate
	 * @param p0 the beginning position in the model
	 * @param p1 the ending position in the model
	 * @returns the location of the end of the range
	 * @exception BadLocationException if the range is invalid
	 */
	protected int drawUnselectedText(Graphics g, int x, int y, int p0, int p1) throws BadLocationException
	   {
	   Document doc = getDocument();
	   Color last = null;

	   int mark = p0;
	   for (; p0 < p1;)
		{
		updateScanner(p0);
		int p = Math.min(lexer.getEndOffset(), p1);
		p = (p <= p0) ? p1 : p;
		Color fg = getForeground(lexer.token);
		if (fg != last && last != null)
		   {
		   // color change, flush what we have
		   g.setColor(last);
		   Segment text = getLineBuffer();
		   doc.getText(mark, p0 - mark, text);
		   x = Utilities.drawTabbedText(text, x, y, g, this, mark);
		   mark = p0;
		   }
		last = fg;
		p0 = p;
		}

	   // flush remaining
	   g.setColor(last);
	   Segment text = getLineBuffer();
	   doc.getText(mark, p1 - mark, text);
	   x = Utilities.drawTabbedText(text, x, y, g, this, mark);

	   return x;
	   }

	/**
	 * Renders using the given rendering surface and area
	 * on that surface.  This is implemented to invalidate
	 * the lexical scanner after rendering so that the next
	 * request to drawUnselectedText will set a new range
	 * for the scanner.
	 *
	 * @param g the rendering surface to use
	 * @param a the allocated region to render into
	 *
	 * @see View#paint
	 */
	public void paint(Graphics g, Shape a)
	   {
	   super.paint(g, a);
	   lexerValid = false;
	   }

	/**
	 * Update the scanner (if necessary) to point to the appropriate
	 * token for the given start position needed for rendering.
	 */
	void updateScanner(int p)
	   {
	   try
		{
		if (!lexerValid)
		   {
		   JavaDocument doc = (JavaDocument) getDocument();
		   lexer.setRange(doc.getScannerStart(p), doc.getLength());
		   lexerValid = true;
		   }
		while (lexer.getEndOffset() <= p)
		   {
		   lexer.scan();
		   }
		}
	   catch (Throwable e)
		{
		}
	   }
	}

   /**
    * Constructs a set of styles to represent java lexical
    * tokens.  By default there are no colors or fonts specified.
    */
   public JavaContext()
	{
	super();

	Style root = getStyle(DEFAULT_STYLE);
	tokenStyles = new Style[Token.MaximumScanValue + 1];
	Token[] tokens = Token.all;

	for (int i = 0; i < tokens.length; i++)
	   {
	   Token t = tokens[i];
	   Style parent = getStyle(t.getCategory());
	   if (parent == null)
		{
		parent = addStyle(t.getCategory(), root);
		}
	   Style s = addStyle(null, parent);
	   s.addAttribute(Token.TokenAttribute, t);
	   tokenStyles[t.getScanValue()] = s;
	   }
	}

   public View create(Element elem)
	{
	return new JavaView(elem);
	}

   /**
    * Fetch the font to use for a lexical
    * token with the given scan value.
    */
   public Font getFont(int code)
	{
	if (tokenFonts == null)
	   {
	   tokenFonts = new Font[Token.MaximumScanValue + 1];
	   }

	if (code < tokenFonts.length)
	   {
	   Font f = tokenFonts[code];
	   if (f == null)
		{
		Style s = tokenStyles[code];
		f = getFont(s);
		}
	   return f;
	   }

	return null;
	}

   /**
    * Fetch the foreground color to use for a lexical
    * token with the given value.
    */
   public Color getForeground(int code)
	{
	if (tokenColors == null)
	   {
	   tokenColors = new Color[Token.MaximumScanValue + 1];
	   }

	if ((code >= 0) && (code < tokenColors.length))
	   {
	   Color c = tokenColors[code];
	   if (c == null)
		{
		Style s = tokenStyles[code];
		c = StyleConstants.getForeground(s);
		}
	   return c;
	   }

	return Color.black;
	}

   /**
    * Fetches the attribute set to use for the given
    * scan code.  The set is stored in a table to
    * facilitate relatively fast access to use in
    * conjunction with the scanner.
    */
   public Style getStyleForScanValue(int code)
	{
	if (code < tokenStyles.length)
	   {
	   return tokenStyles[code];
	   }
	return null;
	}
   }
