/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing.text;

import de.hillenbrand.swing.text.*;
import java.awt.Color;
import java.io.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
* A document to represent text in the form of the
* java programming language.  This is quite primitive
* in that it simply provides support for lexically
* analyzing the text.
* 
* @author  Timothy Prinzing
* @version 1.2 05/04/98
*/

public class JavaDocument extends UndoableDocument {
	static final Object CommentAttribute = new AttributeKey();
	
	static class AttributeKey {
		
		private AttributeKey()  {
		}

		public String toString()  {
			return "comment";
		}

	}

	/**
	* Class to provide InputStream functionality from a portion of a
	* Document.  This really should be a Reader, but not enough
	* things use it yet.
	*/

	class DocumentInputStream extends InputStream {
		Segment segment;
		int p0;
		int p1;
		int pos;
		int index;
		public DocumentInputStream(int p0, int p1)  {
			this.segment = new Segment();
			this.p0 = p0;
			this.p1 = Math.min(getLength(), p1);
			pos = p0;
			try {
			loadSegment();
			} catch (IOException ioe) {
			throw new Error("unexpected: " + ioe);
			}
		}

		void loadSegment() throws IOException {
			try {
			int n = Math.min(1024, p1 - pos);
			getText(pos, n, segment);
			pos += n;
			index = segment.offset;
			} catch (BadLocationException e) {
			throw new IOException("Bad location");
			}
		}

		/**
		* Reads the next byte of data from this input stream. The value
		* byte is returned as an <code>int</code> in the range
		* <code>0</code> to <code>255</code>. If no byte is available
		* because the end of the stream has been reached, the value
		* <code>-1</code> is returned. This method blocks until input data
		* is available, the end of the stream is detected, or an exception
		* is thrown.
		* <p>
		* A subclass must provide an implementation of this method.
		* 
		* @return     the next byte of data, or <code>-1</code> if the end of the
		* stream is reached.
		* @exception  IOException  if an I/O error occurs.
		* @since      JDK1.0
		*/

		public int read() throws IOException {
			if (index >= segment.offset + segment.count) {
			if (pos >= p1) {
			 // no more data
			 return -1;
			}
			loadSegment();
			}
			return segment.array[index++];
		}

	}

	static class LocalEnvironment extends sun.tools.java.Environment {
		
		public void error(Object source, int where, String err, Object arg1, Object arg2, Object arg3)  {
			// should do something useful... 
			//System.err.println(err);
			//System.err.println("location: " + where);
		}

	}

	public class Scanner extends sun.tools.java.Scanner {
		DocumentInputStream dis = new DocumentInputStream(0, getLength());
		int p0;
		
		Scanner() throws IOException {
			super(new LocalEnvironment());
			// JDK 1.2: super(new LocalEnvironment(), super(new LocalEnvironment()););
			scanComments = true;
		}

		/**
		* This fetches the ending location of the current
		* token in the document.
		*/

		public final int getEndOffset()  {
			int endOffs = (int) (getEndPos() & MAXFILESIZE);
			return p0 + endOffs;
		}

		/**
		* This fetches the starting location of the current
		* token in the document.
		*/

		public final int getStartOffset()  {
			int begOffs = (int) (pos & MAXFILESIZE);
			return p0 + begOffs;
		}

		/**
		* Sets the range of the scanner.  This should be called
		* to reinitialize the scanner to the desired range of
		* coverage.
		*/

		public void setRange(int p0, int p1) throws IOException {
			useInputStream(new DocumentInputStream(p0, p1));
			this.p0 = p0;
		}

	}

	/**
	* Create a lexical analyzer for this document.
	*/

	public Scanner createScanner()  {
		Scanner s;
		
		try
			{
			s = new Scanner();
			}
		catch (IOException e) { s = null; }
		
		return s;
	}

	/**
	* Fetch a reasonable location to start scanning
	* given the desired start location.  This allows
	* for adjustments needed to accomodate multiline
	* comments.
	*/

	public int getScannerStart(int p)  {
		Element elem = getDefaultRootElement();
		int lineNum = elem.getElementIndex(p);
		Element line = elem.getElement(lineNum);
		AttributeSet a = line.getAttributes();
		
		while (a.isDefined(CommentAttribute) && lineNum > 0) 
			{
			lineNum -= 1;
			line = elem.getElement(lineNum);
			a = line.getAttributes();
			}
		
		return line.getStartOffset();
	}

	/**
	* Updates document structure as a result of text insertion.  This
	* will happen within a write lock.  The superclass behavior of
	* updating the line map is executed followed by marking any comment
	* areas that should backtracked before scanning.
	* 
	* @param chng the change event
	* @param attr the set of attributes
	*/

	protected void insertUpdate(DefaultDocumentEvent chng, AttributeSet attr)  {
		super.insertUpdate(chng, attr);
		
		// update comment marks
		Element root = getDefaultRootElement();
		DocumentEvent.ElementChange ec = chng.getChange(root);
		if (ec != null) {
		    Element[] added = ec.getChildrenAdded();
		    boolean inComment = false;
		    for (int i = 0; i < added.length; i++) {
			Element elem = added[i];
			int p0 = elem.getStartOffset();
			int p1 = elem.getEndOffset();
			String s;
			try {
			    s = getText(p0, p1 - p0);
			} catch (BadLocationException bl) {
			    s = null; 
			}
			if (inComment) {
			    MutableAttributeSet a = (MutableAttributeSet) elem.getAttributes();
			    a.addAttribute(CommentAttribute, CommentAttribute);
			    int index = s.indexOf("*/");
			    if (index >= 0) {
				// found an end of comment, turn off marks
				inComment = false;
			    }
			} else {
			    // scan for multiline comment
			    int index = s.indexOf("/*");
			    if (index >= 0) {
				// found a start of comment, see if it spans lines
				index = s.indexOf("*/", index);
				if (index < 0) {
				    // it spans lines
				    inComment = true;
				}
			    }
			}
		    }
		}
	}

	/**
	* Updates any document structure as a result of text removal.
	* This will happen within a write lock.  The superclass behavior of
	* updating the line map is executed followed by placing a lexical
	* update command on the analyzer queue.
	* 
	* @param chng the change event
	*/

	protected void removeUpdate(DefaultDocumentEvent chng)  {
		super.removeUpdate(chng);
	}

}
