/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing.text;

import de.hillenbrand.swing.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.undo.*;

/**
 * A document to represent text in the form of the
 * java programming language.  This is quite primitive
 * in that it simply provides support for lexically
 * analyzing the text.
 *
 * @author  Timothy Prinzing
 * @version 1.2 05/04/98
 */
public class UndoableDocument extends PlainDocument
   {
   private UndoManager undoManager = new UndoManager();
   private UndoAction undoAction = new UndoAction();
   private RedoAction redoAction = new RedoAction();
   private UndoableEditListener listener = new UndoHandler();

   class RedoAction extends AbstractAction
	{
	public RedoAction()
	   {
	   super("redo");
	   setEnabled(false);
	   }

	public void actionPerformed(java.awt.event.ActionEvent arg0)
	   {
	   try
		{
		undoManager.redo();
		}
	   catch (Throwable x)
		{
		System.err.println(x);
		}

	   update();
	   undoAction.update();
	   }

	protected void update()
	   {
	   setEnabled(undoManager.canRedo());
	   }
	}

   class UndoAction extends AbstractAction
	{
	public UndoAction()
	   {
	   super("undo");
	   setEnabled(false);
	   }

	public void actionPerformed(java.awt.event.ActionEvent arg0)
	   {
	   try
		{
		undoManager.undo();
		}
	   catch (Throwable x)
		{
		System.err.println(x);
		}

	   update();
	   redoAction.update();
	   }

	protected void update()
	   {
	   setEnabled(undoManager.canUndo());
	   }
	}

   class UndoHandler implements UndoableEditListener
	{
	public void undoableEditHappened(UndoableEditEvent e)
	   {
	   undoManager.addEdit(e.getEdit());
	   undoAction.update();
	   redoAction.update();
	   }
	}

   public UndoableDocument()
	{
	super(new GapContent(1024));

	// add listener
	addUndoableEditListener(listener);

	// set tab width
	putProperty(PlainDocument.tabSizeAttribute, new Integer(HEditorPane.getDefaultTabSize()));
	}

   public boolean canRedo()
	{
	return undoManager.canRedo();
	}

   public boolean canUndo()
	{
	return undoManager.canUndo();
	}

   public String getText()
	{
	try
	   {
	   return getText(0, getLength());
	   }
	catch (BadLocationException e)
	   {
	   System.err.println(getClass().getName() + ".getText(): " + e);
	   }
	return "GET TEXT FAILED -> SORRY.";
	}

   public void redo()
	{
	undoManager.redo();

	redoAction.update();
	undoAction.update();
	}

   public void setText(String text)
	{
	removeUndoableEditListener(listener);
	try
	   {
	   remove(0, getLength());
	   insertString(0, text, null);
	   }
	catch (BadLocationException e)
	   {
	   System.err.println(getClass().getName() + ".setText(): " + e);
	   }
	addUndoableEditListener(listener);
	}

   public void undo()
	{
	undoManager.undo();

	undoAction.update();
	redoAction.update();
	}

   public void discardAllEdits()
	{
	undoManager.discardAllEdits();
	}
   }
