/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.util;

import java.lang.reflect.*;
import java.util.*;

/**
 * Some utilities concerning arrays.
 */
public class ArrayUtils
   {
   public static final Object[] NULLARRAY = new Object[]
	{
	null
	};

   private ArrayUtils()
	{
	}

   /**
    * Find the index of object o in the array a using the comparator c. Returns -1 if o is not in a.
    * @param <T>
    * @param array
    * @param object
    * @param comp
    * @return
    */
   public static synchronized <T> int indexOf(T[] array, T object, Comparator<? super T> comp)
	{
	if (array == null)
	   {
	   return -1;
	   }

	for (int i = 0; i < array.length; i++)
	   {
	   if (comp == null && array[i].equals(object))
		{
		return i;
		}
	   else if (comp != null && comp.compare(array[i], object) == 0)
		{
		return i;
		}
	   }

	return -1;
	}

   public static String toString(Object[] data)
	{
	return toString(data, "\0");
	}

   public static String toString(Object[] data, String delimiter)
	{
	final StringBuffer result = new StringBuffer();
	for (int i = 0; i < data.length; ++i)
	   {
	   result.append(data[i].toString());
	   if (i + 1 < data.length)
		{
		result.append(delimiter);
		}
	   }
	return result.toString();
	}

   public static String toString(Object[] data, String delimiter, String methodNameToCall) throws NoSuchMethodException, IllegalAccessException, IllegalArgumentException, InvocationTargetException
	{
	final ArrayList<Object> a = new ArrayList<Object>();
	for (Object o : data)
	   {
	   final Method m = o.getClass().getMethod(methodNameToCall, new Class[0]);
	   final Object callResult = m.invoke(o, new Object[0]);
	   a.add(callResult);
	   }
	return toString(a.toArray(new Object[0]), delimiter);
	}

   /**
    * Create an array from a list of arguments.
    * @param <T>
    * @param data
    * @return
    */
   public static <T> T[] toArray(final T... data)
	{
	return data;
	}

   /**
    * Converts an array of any type into a String array.
    * @param <T>
    * @param data
    * @return
    */
   public static <T> String[] toStringArray(final T[] data)
	{
	if (data == null)
	   {
	   return new String[0];
	   }
	final String[] result = new String[data.length];
	for (int i = 0; i < data.length; ++i)
	   {
	   result[i] = data[i] != null ? data[i].toString() : null;
	   }
	return result;
	}

   /**
    * Converts the field of a class into a string array.
    * @param c
    * @return
    */
   public static String[] toStringArray(Class c)
	{
	final ArrayList<String> result = new ArrayList<String>();

	try
	   {
	   final Field[] fields = c.getDeclaredFields();
	   for (Field f : fields)
		{
		final Class type = f.getType();
		if (type.getName().equals(c.getName()))
		   {
		   result.add("" + f.get(null));
		   }
		}
	   }
	catch (Exception x)
	   {
	   throw new RuntimeException("Class '" + c + "' could not be converted to String array.", x);
	   }

	return result.toArray(new String[0]);
	}

   /**
    * Joins two arrays of any type into one array.
    * @param <T>
    * @param array1
    * @param array2
    * @return
    */
   public static <T> T[] join(T[] array1, T[] array2)
	{
	if (array1 == null && array2 == null)
	   {
	   throw new RuntimeException("Can't join two null arrays.");
	   }
	if (array1 == null)
	   {
	   return array2;
	   }
	if (array2 == null)
	   {
	   return array1;
	   }
	@SuppressWarnings("unchecked")
	final T[] result = (T[]) java.lang.reflect.Array.newInstance(array1.getClass().getComponentType(), array1.length + array2.length);
	System.arraycopy(array1, 0, result, 0, array1.length);
	System.arraycopy(array2, 0, result, array1.length, array2.length);
	return result;
	}
   }
