/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.util;

import java.awt.*;
import java.awt.image.*;

public final class ColorUtils
   {
   public static BufferedImage createTranslucentImage(int width, int height)
	{
	return GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage(width, height, Transparency.TRANSLUCENT);
	}

   public static Color deriveColorAlpha(Color base, int alpha)
	{
	if (base == null)
	   {
	   System.err.println("ColorUtils.deriveColorAlpha(): ERROR: base == null");
	   base = Color.red;
	   }

	return new Color(base.getRed(), base.getGreen(), base.getBlue(), alpha);
	}

   /**
    * Derives a color by adding the specified offsets to the base color's
    * hue, saturation, and brightness values.  The resulting hue, saturation,
    * and brightness values will be contrained to be between 0 and 1.
    *
    * @param base the color to which the HSV offsets will be added
    * @param dH the offset for hue
    * @param dS the offset for saturation
    * @param dB the offset for brightness
    * @return Color with modified HSV values
    *
    */
   public static final Color deriveColorHSB(Color base, float dH, float dS, float dB)
	{
	if (base == null)
	   {
	   System.err.println("ColorUtils.deriveColorHSB(): ERROR: base == null");
	   base = Color.red;
	   }

	final float hsb[] = Color.RGBtoHSB(base.getRed(), base.getGreen(), base.getBlue(), null);
	hsb[0] += dH;
	hsb[1] += dS;
	hsb[2] += dB;

	return Color.getHSBColor(hsb[0] < 0 ? 0 : (hsb[0] > 1 ? 1 : hsb[0]), hsb[1] < 0 ? 0 : (hsb[1] > 1 ? 1 : hsb[1]), hsb[2] < 0 ? 0 : (hsb[2] > 1 ? 1 : hsb[2]));
	}

   public static final Color makeDarker(Color c, int value)
	{
	if (c == null)
	   {
	   return c;
	   }

	int r = c.getRed();
	int g = c.getGreen();
	int b = c.getBlue();

	r = Math.max(0, r - value);
	g = Math.max(0, g - value);
	b = Math.max(0, b - value);

	return new Color(r, g, b);
	}
   }
