/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.util;

import java.io.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Utilities that help handling files and directories.
 */
public class FileUtils
   {
   public static String changeSuffix(String filename, String newSuffix)
	{
	if (newSuffix != null && newSuffix.startsWith("."))
	   {
	   newSuffix = newSuffix.substring(1);
	   }
	return FileUtils.removeSuffix(filename) + "." + newSuffix;
	}

   public static File changeSuffix(File file, String newSuffix)
	{
	if (newSuffix != null && newSuffix.startsWith("."))
	   {
	   newSuffix = newSuffix.substring(1);
	   }
	final String filename = file.getName();
	return new File(FileUtils.removeSuffix(filename) + "." + newSuffix);
	}

   public static boolean copyFile(File source, File destination)
	{
	boolean result = true;

	try
	   {
	   final FileInputStream in = new FileInputStream(source);
	   final FileOutputStream out = new FileOutputStream(destination);

	   byte[] buffer = new byte[1024];
	   while (true)
		{
		int read = in.read(buffer);
		if (read == -1)
		   {
		   break;
		   }
		out.write(buffer, 0, read);
		}

	   out.close();
	   in.close();
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }

	return result;
	}

   public static boolean copyFile(String source, String destination)
	{
	return FileUtils.copyFile(new File(source), new File(destination));
	}

   public static boolean copyFileIfDifferent(File source, File destination)
	{
	try
	   {
	   final FileInputStream in = new FileInputStream(source);
	   byte[] inBuffer = new byte[(int) source.length()];
	   in.read(inBuffer);
	   in.close();

	   if (inBuffer.length == (int) destination.length())
		{
		final FileInputStream destIn = new FileInputStream(destination);
		final byte[] destBuffer = new byte[inBuffer.length];
		destIn.read(destBuffer);
		destIn.close();
		for (int i = 0; i < inBuffer.length; ++i)
		   {
		   if (inBuffer[i] != destBuffer[i])
			{
			return copyFileIfDifferent2(inBuffer, destination);
			}
		   }
		}
	   else
		{
		return copyFileIfDifferent2(inBuffer, destination);
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }

	return false;
	}

   private static boolean copyFileIfDifferent2(byte[] buffer, File destination)
	{
	try
	   {
	   final FileOutputStream out = new FileOutputStream(destination);
	   out.write(buffer, 0, buffer.length);
	   out.close();
	   return true;
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   return false;
	   }
	}

   /**
    * Deletes a directory recursively. If the argument is a file, it will be deleted.
    *
    * @param	directory the directory to delete
    * @return	true on success, false otherwise
    */
   public static boolean deleteDirectory(File directory)
	{
	if (!directory.exists())
	   {
	   return true;
	   }

	try
	   {
	   if (!directory.isDirectory())
		{
		return directory.delete();
		}

	   final String[] files = directory.list();
	   for (int i = 0; i < files.length; i++)
		{
		final File f = new File(directory, files[i]);
		FileUtils.deleteDirectory(f);
		}

	   return directory.delete();
	   }
	catch (SecurityException x)
	   {
	   return false;
	   }
	}

   /**
    * Deletes a directory recursively. If the argument is a file, it will be deleted.
    *
    * @param	directory the directory to delete
    * @return	true on success, false otherwise
    */
   public static boolean deleteDirectory(String directory)
	{
	return deleteDirectory(new File(directory));
	}

   public static boolean doesExist(String filename)
	{
	File f = new File(filename);
	return f.exists();
	}

   /**
    * Deletes all files and subdiurectories from directory recursively. The directory itself will NOT be deleted.
    *
    * @param	directory the directory to delete
    * @return	true on success, false otherwise
    */
   public static boolean emptyDirectory(File directory)
	{
	if (!directory.exists())
	   {
	   return false;
	   }
	if (!directory.isDirectory())
	   {
	   return false;
	   }

	boolean result = true;

	try
	   {
	   final String[] files = directory.list();
	   for (int i = 0; i < files.length; i++)
		{
		final File f = new File(directory, files[i]);
		boolean ok = FileUtils.deleteDirectory(f);
		if (!ok)
		   {
		   result = false;
		   }
		}
	   return result;
	   }
	catch (SecurityException x)
	   {
	   return false;
	   }
	}

   /**
    * Deletes all files and subdiurectories from directory recursively. The directory itself will NOT be deleted.
    *
    * @param	directory the directory to delete
    * @return	true on success, false otherwise
    */
   public static boolean emptyDirectory(String directory)
	{
	return deleteDirectory(new File(directory));
	}

   public static String ensureSuffix(String filename, String suffix)
	{
	if (filename.endsWith(suffix))
	   {
	   return filename;
	   }
	else
	   {
	   return changeSuffix(filename, suffix);
	   }
	}

   public static File ensureSuffix(File file, String suffix)
	{
	if (file.getName().endsWith(suffix))
	   {
	   return file;
	   }
	else
	   {
	   return changeSuffix(file, suffix);
	   }
	}

   /**
    * Returns the System's file encoding. This is derived from System.getProperty("file.encoding"). If this is not set, it will return "8859_1";
    * @return
    */
   public static String getDefaultEncoding()
	{
	final String encoding = System.getProperty("file.encoding");
	if (encoding != null)
	   {
	   return encoding;
	   }
	else
	   {
	   return "8859_1";
	   }
	}

   /**
    * Returns all files with the suffix in the directory and its subsirectories.
    * This method is not case sensitive (for the suffix).
    * Please do not add a "." for the suffix, just the chars!
    * The result is sorted by name.
    * @param directory
    * @param suffix
    * @return
    */
   public static String[] getFiles(String directory, String suffix)
	{
	return getFiles(new File(directory), suffix);
	}

   /**
    * Returns all files with the suffix in the directory and its subsirectories.
    * This method is not case sensitive (for the suffix).
    * Please do not add a "." for the suffix, just the chars!
    * The result is sorted by name.
    * @param directory
    * @param suffix
    * @return
    */
   public static String[] getFiles(File directory, String suffix)
	{
	// init
	suffix = "." + suffix.toLowerCase();

	// the files
	final TreeSet<String> data = new TreeSet<String>();

	// the directories to scan
	final Vector<File> todo = new Vector<File>();

	// check if the argument is a directory
	if (directory.exists() && directory.isDirectory())
	   {
	   todo.addElement(directory);
	   }
	else
	   {
	   System.err.println("FileUtils.getFiles(): '" + directory + "' is not a directory");
	   }

	while (todo.size() > 0)
	   {
	   // get the first
	   directory = todo.elementAt(0);
	   todo.removeElementAt(0);
	   // list all files and process the file names
	   final String[] files = directory.list();
	   for (int i = 0; files != null && i < files.length; ++i)
		{
		final File f = new File(directory, files[i]);
		if (f.isDirectory())
		   {
		   todo.addElement(f);
		   }
		else if (files[i].toLowerCase().endsWith(suffix))
		   {
		   data.add(f.toString());
		   }
		}
	   }

	return data.toArray(new String[0]);
	}

   public static String[] getSubdirectories(String dirname)
	{
	final File dir = new File(dirname);

	final String[] subdirs = dir.list(new FilenameFilter()
	   {
	   public boolean accept(File d, String f)
		{
		File test = new File(d, f);
		return test.isDirectory();
		}
	   });
	if (subdirs == null)
	   {
	   return new String[0];
	   }

	Arrays.sort(subdirs);
	return subdirs;
	}

   public static String toString(File file)
	{
	try
	   {
	   return StringUtils.readStringFromFile(file);
	   }
	catch (IOException ex)
	   {
	   Logger.getLogger(FileUtils.class.getName()).log(Level.SEVERE, null, ex);
	   return ex.toString();
	   }
	}

   public static String toString(String filename)
	{
	try
	   {
	   return StringUtils.readStringFromFile(filename);
	   }
	catch (IOException ex)
	   {
	   Logger.getLogger(FileUtils.class.getName()).log(Level.SEVERE, null, ex);
	   return ex.toString();
	   }
	}


   public static String removeSuffix(String filename)
	{
	// get index of "."
	int index = filename.lastIndexOf(".");

	// create result
	if (index == -1)
	   {
	   return filename;
	   }
	else
	   {
	   return filename.substring(0, index);
	   }
	}

   /**
    * Writes the text into the file.
    * @param file
    * @param text
    * @return
    */
   public static boolean writeToFileNoException(File file, String text)
	{
	boolean result = false;

	try
	   {
	   final PrintWriter out = new PrintWriter(new FileWriter(file));
	   out.print(text);
	   out.close();
	   result = true;
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }

	return result;
	}

   public static void writeToFile(File file, String text) throws IOException
	{
	final PrintWriter out = new PrintWriter(new FileWriter(file));
	out.print(text);
	out.close();
	}
   }
