/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.util;

import java.io.*;
import java.net.*;
import java.util.*;

public class StringUtils
   {
   public static final String DEFAULT_DELIMITER = "\0";

   private StringUtils()
	{
	}

   /**
    * Converts an array of strings into a single string.
    * @param s
    * @return
    */
   public static synchronized String toString(final String[] s)
	{
	return toString(s, DEFAULT_DELIMITER);
	}

   /**
    * Converts an array of strings into a single string using the given delimiter.
    * @param s
    * @param delimiter
    * @return
    */
   public static synchronized String toString(final String[] s, final String delimiter)
	{
	if (s == null)
	   {
	   return "";
	   }

	int i, len = s.length;
	String result = "";
	for (i = 0; i < len; i++)
	   {
	   result += s[i];
	   if (i < len - 1)
		{
		result += delimiter;
		}
	   }

	return result;
	}

   /**
    * Converts an array of strings into a single string.
    * @param s
    * @return
    * @deprecated
    */
   public static synchronized String convertToString(final String[] s)
	{
	return toString(s, DEFAULT_DELIMITER);
	}

   /**
    * Converts an array of strings into a single string using the given delimiter.
    * @param s
    * @param delimiter
    * @return
    * @deprecated
    */
   public static synchronized String convertToString(final String[] s, final String delimiter)
	{
	return toString(s, delimiter);
	}

   /**
    * Counts the number of tabs in the text.
    * @param text
    * @return
    */
   public static int countTabs(String text)
	{
	int count = 0;
	int index = 0;

	while (true)
	   {
	   index = text.indexOf("\t", index);
	   if (index == -1)
		{
		break;
		}
	   ++index;
	   ++count;
	   }

	return count;
	}

   public static synchronized int indexOf(String[] array, String s)
	{
	if (array == null)
	   {
	   return -1;
	   }
	int i, len = array.length;
	for (i = 0; i < len; i++)
	   {
	   if (array[i] != null && array[i].equals(s))
		{
		return i;
		}
	   }
	return -1;
	}

   /**
    * Inserts a new line when reaching the word after maxCharsPerLine.
    * @param s
    * @param maxCharsPerLine
    * @return
    */
   public static String insertNewline(String s, int maxCharsPerLine)
	{
	final int len = s.length();
	final StringBuffer result = new StringBuffer(len + 10);

	int count = 0;
	for (int i = 0; i < len; ++i)
	   {
	   final char c = s.charAt(i);
	   result.append(c);
	   count++;
	   if (c == '\n')
		{
		count = 0;
		}
	   if (count >= maxCharsPerLine && c == ' ')
		{
		result.append('\n');
		count = 0;
		}
	   }

	return result.toString();
	}

   /**
    * Shortens a string to length characters and indicated this with dots.
    * @param s
    * @param length
    * @return
    */
   public static String limitStringLength(String s, int length)
	{
	return limitStringLength(s, length, true);
	}

   /**
    * Shortens a string to length characters and indicated this with dots.
    * @param s
    * @param length
    * @param addDots
    * @return
    */
   public static String limitStringLength(String s, int length, boolean addDots)
	{
	if (s == null)
	   {
	   return "";
	   }

	return addDots
		  ? (s.length() < length) ? s : s.substring(0, length - 1) + "..."
		  : (s.length() < length) ? s : s.substring(0, length);

	}

   /**
    * Parses a string array out of a string using the default separator ("\0").
    * @param s
    * @return
    * @deprecated
    */
   public static synchronized String[] parseArrayFromString(final String s)
	{
	return toArray(s, DEFAULT_DELIMITER);
	}

   /**
    * Parses a string array out of a string using the given separator.
    * @param s
    * @param separator
    * @return
    * @deprecated
    */
   public static synchronized String[] parseArrayFromString(final String s, final String separator)
	{
	return toArray(s, separator);
	}

   /**
    * Converts a string into an array
    * @param s
    * @return
    */
   public static synchronized String[] toArray(final String s)
	{
	return toArray(s, DEFAULT_DELIMITER);
	}

   /**
    * Converts a string into an array using the given separator.
    * @param s
    * @param separator
    * @return
    */
   public static synchronized String[] toArray(final String s, final String separator)
	{
	// test for null
	if (s == null)
	   {
	   return new String[0];
	   }

	// create tokenizer and result array
	final StringTokenizer tok = new StringTokenizer(s, separator);
	final int count = tok.countTokens();
	final String[] result = new String[count];

	// get the tokens
	for (int i = 0; i < count; ++i)
	   {
	   result[i] = tok.nextToken();
	   }

	// and return the array
	return result;
	}

   /**
    * Removes the String itemToRemove from the array. If itemToRemove is more than once in the array, only the first will be removed.
    * @param array
    * @param itemToRemove
    * @return
    */
   public static String[] removeFromArray(String[] array, String itemToRemove)
	{
	if (array == null || array.length == 0)
	   {
	   return array;
	   }
	if (itemToRemove == null | itemToRemove.length() == 0)
	   {
	   return array;
	   }

	int index = StringUtils.indexOf(array, itemToRemove);
	if (index == -1)
	   {
	   return array;
	   }

	String[] result = new String[array.length - 1];
	int count = 0;
	for (int i = 0; i < array.length; ++i)
	   {
	   if (index == i)
		{
		continue;
		}
	   result[count] = array[i];
	   ++count;
	   }

	return result;
	}

   /**
    * Replaces "oldText" with "newText" in the String "text".
    * @param text 
    * @param oldText
    * @param newText
    * @return
    */
   public static String replace(String text, String oldText, String newText)
	{
	int index = 0;

	while (true)
	   {
	   index = text.indexOf(oldText, index);
	   if (index == -1)
		{
		break;
		}
	   text = text.substring(0, index) + newText + text.substring(index + oldText.length());
	   index = index + newText.length();
	   }

	return text;
	}

   public static String readStringFromFile(File file) throws java.io.IOException
	{
	final StringBuffer result = new StringBuffer();
	final BufferedReader in = new BufferedReader(new FileReader(file));
	final char[] buffer = new char[1024];

	while (true)
	   {
	   int read = in.read(buffer, 0, 1024);
	   if (read == -1)
		{
		break;
		}
	   result.append(new String(buffer, 0, read));
	   }
	in.close();

	return result.toString();
	}

   public static String readStringFromFile(String filename) throws java.io.IOException
	{
	return readStringFromFile(new File(filename));
	}

   public static String readStringFromFile(File directory, String filename) throws java.io.IOException
	{
	return readStringFromFile(new File(directory, filename));
	}

   public static String readStringFromFile(String directory, String filename) throws java.io.IOException
	{
	return readStringFromFile(new File(directory, filename));
	}

   public static String readStringFromURL(URL url) throws IOException
	{
	final StringBuffer buffer = new StringBuffer();
	final BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
	String inputLine;

	while ((inputLine = reader.readLine()) != null)
	   {
	   if (buffer.length() > 0)
		{
		buffer.append("\n");
		}
	   buffer.append(inputLine);
	   }
	reader.close();

	return buffer.toString();
	}

   public static String readStringFromURL(String url) throws MalformedURLException, IOException
	{
	return readStringFromURL(new URL(url));
	}
   }
